﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;

namespace HP.ETL.Data
{
	public class CultureManager
    {
        
        public const string AutoDetectCulture = "Detect,Detect";
        
        public static string[] SupportedCultures = new string[] {
                "en-US,en-US"};
        
        public static void Initialize()
        {
        }
        
        public static string ResolveEmbeddedResourceName(string resourceName, string culture)
        {
            return ResolveEmbeddedResourceName(typeof(CultureManager).Assembly, resourceName, culture);
        }
        
        public static string ResolveEmbeddedResourceName(string resourceName)
        {
            return ResolveEmbeddedResourceName(typeof(CultureManager).Assembly, resourceName, Thread.CurrentThread.CurrentUICulture.Name);
        }
        
        public static string ResolveEmbeddedResourceName(Assembly a, string resourceName, string culture)
        {
            string extension = Path.GetExtension(resourceName);
            string fileName = Path.GetFileNameWithoutExtension(resourceName);
            string localizedResourceName = String.Format("{0}.{1}{2}", fileName, culture.Replace("-", "_"), extension);
            ManifestResourceInfo mri = a.GetManifestResourceInfo(localizedResourceName);
            if (mri == null)
            {
                if (culture.Contains("-"))
                	localizedResourceName = String.Format("{0}.{1}_{2}", fileName, culture.Substring(0, culture.LastIndexOf("-")).Replace("-", "_"), extension);
                else
                	localizedResourceName = String.Format("{0}.{1}_{2}", fileName, culture, extension);
                mri = a.GetManifestResourceInfo(localizedResourceName);
            }
            if (mri == null)
            	localizedResourceName = resourceName;
            return localizedResourceName;
        }
    }
    
    public class GenericHandlerBase
    {
        
        public GenericHandlerBase()
        {
            CultureManager.Initialize();
        }
        
        protected virtual string GenerateOutputFileName(ActionArgs args, string outputFileName)
        {
            args.CommandArgument = args.CommandName;
            args.CommandName = "FileName";
            List<FieldValue> values = new List<FieldValue>();
            values.Add(new FieldValue("FileName", outputFileName));
            args.Values = values.ToArray();
            ActionResult result = ControllerFactory.CreateDataController().Execute(args.Controller, args.View, args);
            foreach (FieldValue v in result.Values)
            	if (v.Name == "FileName")
                {
                    outputFileName = Convert.ToString(v.Value);
                    break;
                }
            return outputFileName;
        }
        
        protected virtual void AppendDownloadTokenCookie()
        {
            HttpContext context = HttpContext.Current;
            string downloadToken = "APPFACTORYDOWNLOADTOKEN";
            HttpCookie tokenCookie = context.Request.Cookies[downloadToken];
            if (tokenCookie == null)
            	tokenCookie = new HttpCookie(downloadToken);
            tokenCookie.Value = String.Format("{0},{1}", tokenCookie.Value, Guid.NewGuid());
            context.Response.AppendCookie(tokenCookie);
        }
    }
}
