﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using HP.ETL.Web;

namespace HP.ETL.WebParts
{
	[ToolboxItemAttribute(false)]
    public partial class AppWebPart : AppWebPartBase
    {
    }
    
    [ToolboxItemAttribute(false)]
    public class AppWebPartBase : WebPart
    {
        
        private const string _ascxPath = "~/_CONTROLTEMPLATES/HP.ETL/AppWebPart/DefaultUserControl.ascx";
        
        public AppWebPartBase() : 
                base()
        {
            ChromeType = PartChromeType.None;
        }
        
        [Personalizable(true)]
        [WebBrowsable(false)]
        public string ApplicationPage
        {
            get
            {
                return ((string)(ViewState["ApplicationPage"]));
            }
            set
            {
                ViewState["ApplicationPage"] = value;
            }
        }
        
        protected override void CreateChildControls()
        {
            string pageName = ApplicationPage;
            string baseFolder = "Pages";
            if (String.IsNullOrEmpty(pageName))
            {
                pageName = "DefaultUserControl";
                baseFolder = "AppWebPart";
            }
            else
            	if (this.DesignMode || SPContext.Current.IsDesignTime)
                {
                    Controls.Add(new LiteralControl(String.Format("<div style=\"height:50px;border:solid 1px silver;margin:8px;padding:8px;\"> Logical" +
                                    " page \'{0}\' will be rendered here at runtime.</div>", pageName)));
                    return;
                }
            string controlPath = _ascxPath.Replace("/AppWebPart/", String.Format("/{0}/", baseFolder)).Replace("/DefaultUserControl.", String.Format("/{0}.", pageName));
            try
            {
                Control control = Page.LoadControl(controlPath);
                ControlBase.LoadPageControl(this, pageName, control);
            }
            catch (Exception ex)
            {
                Controls.Add(new LiteralControl(ex.Message));
            }
        }
        
        public override EditorPartCollection CreateEditorParts()
        {
            List<EditorPart> editors = new List<EditorPart>();
            EditorPart part = new AppEditorPart();
            part.ID = (this.ID + "_WebAppEditor");
            editors.Add(part);
            EditorPartCollection list = base.CreateEditorParts();
            return new EditorPartCollection(list, editors);
        }
    }
    
    public class AppEditorPart : EditorPart
    {
        
        private AppSelector _selector;
        
        protected override void CreateChildControls()
        {
            this.Title = "Settings";
            base.CreateChildControls();
            _selector = new AppSelector();
            _selector.ID = "AppSelector";
            if (SPContext.Current.IsDesignTime)
            	Controls.Add(new LiteralControl("<div style=\"width:90%;\">"));
            Controls.Add(_selector);
            if (SPContext.Current.IsDesignTime)
            	Controls.Add(new LiteralControl("</div>"));
            _selector.ApplicationPage = ((AppWebPart)(WebPartToEdit)).ApplicationPage;
        }
        
        public override bool ApplyChanges()
        {
            EnsureChildControls();
            ((AppWebPart)(WebPartToEdit)).ApplicationPage = _selector.ApplicationPage;
            return true;
        }
        
        public override void SyncChanges()
        {
            EnsureChildControls();
        }
    }
}
