﻿


CREATE procedure [ETL].[AlterProjectAccess]
	@ProjectName	nvarchar(20)
as
/*******************************************************************************************
Description:	Alters project acess Steps 5-8 from the 
				Configuring Project ETL Server Access document

			  
Parameters:
	@ProjectName - Name of the project being configured

Assumptions:	1.	Schema exists
				2.	_DEV user exists
				3.	_ETL user exists
				4.	_DEV owns schema
				5.	_ETL has credential
				6.	_ETL is proxy

Process:		1.	Create Project Login
				2.	Assing Project Login to sysadmin
				3.	Create ETL database Project User for Project Login
				4.	Change ownership of project schema to Project User
				5.	Grant schema permissions to _ETL User
				6.	Drop Project_DEV User from ETL database
				7.	Drop Project_DEV Login from server

*******************************************************************************************/



declare		
	@UseMaster nvarchar(50)		
	,@LoginExistsFlag bit
	,@CheckLoginExists nvarchar(500)
	,@CreateNewLogin nvarchar(1000)
	,@AlterServerRole nvarchar(1000)
	,@UseETL nvarchar(50)
	,@CreateNewUser nvarchar(500)
	,@ChangeOwner nvarchar(500)
	,@DropOldUser nvarchar(500)
	,@DropOldLogin nvarchar(500)
	,@GrantAlter nvarchar(500)
	,@GrantDelete nvarchar(500)
	,@GrantExecute nvarchar(500)
	,@GrantInsert nvarchar(500)
	,@GrantReferences nvarchar(500)
	,@GrantSelect nvarchar(500)
	,@GrantUpdate nvarchar(500)
	,@GrantViewDef nvarchar(500)

--check for login existence
set @CheckLoginExists = 'use [master]; select @LoginExistsFlag = case when exists (select 1 from sys.syslogins where name = ''VHAMASTER\' + @ProjectName + ''') then 1 else 0 end;';
exec sp_executesql @CheckLoginExists, N'@LoginExistsFlag bit output',@LoginExistsFlag = @LoginExistsFlag output;

If @LoginExistsFlag = 0

begin

	--create login
	set @UseMaster = 'Use [master]; exec(@CreateNewLogin)';
	set @CreateNewLogin = 'CREATE LOGIN [VHAMASTER\'+@ProjectName+'] FROM WINDOWS WITH DEFAULT_DATABASE=[ETL], DEFAULT_LANGUAGE=[us_english]'
	exec sp_executesql @UseMaster, N'@CreateNewLogin nvarchar(1000)',@CreateNewLogin;

	--assign sysadmin role to login
	set @UseMaster = 'Use [master]; exec(@AlterServerRole)';
	set @AlterServerRole = 'ALTER SERVER ROLE [sysadmin] ADD MEMBER [VHAMASTER\'+@ProjectName+']'
	exec sp_executesql @UseMaster, N'@AlterServerRole nvarchar(1000)',@AlterServerRole;

	--create new user for ETL database
	set @UseETL = 'Use [ETL]; exec(@CreateNewUser)';
	set @CreateNewUser = 'CREATE USER [u'+@ProjectName+'] FOR LOGIN [VHAMASTER\'+@ProjectName+'] WITH DEFAULT_SCHEMA=['+@ProjectName+']'
	exec sp_executesql @UseETL, N'@CreateNewUser nvarchar(1000)',@CreateNewUser;

	--change ownership of project schema to new user
	set @UseETL = 'Use [ETL]; exec(@ChangeOwner)';
	set @ChangeOwner ='ALTER AUTHORIZATION ON SCHEMA::'+@ProjectName+' TO u'+@ProjectName+';'
	exec sp_executesql @UseETL, N'@ChangeOwner nvarchar(1000)',@ChangeOwner;

	--drop old user from ETL database
	set @UseETL = 'Use [ETL]; exec(@DropOldUser)';
	set @DropOldUser = 'DROP USER [u'+@ProjectName+'_DEV];'
	exec sp_executesql @UseETL, N'@DropOldUser nvarchar(500)',@DropOldUser;

	--drop old login from server
	set @UseMaster = 'Use [master]; exec(@DropOldLogin)';
	set @DropOldLogin = 'DROP LOGIN [VHAMASTER\'+@ProjectName+'_DEV];'
	exec sp_executesql @UseMaster, N'@DropOldLogin nvarchar(500)',@DropOldLogin;

	--grant permissions on schema to ETL user due to new schema owner
	set @UseETL = 'Use [ETL]; exec(@GrantAlter)';
	set @GrantAlter = 'GRANT ALTER ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantAlter nvarchar(1000)',@GrantAlter;

	set @UseETL = 'Use [ETL]; exec(@GrantDelete)';
	set @GrantDelete = 'GRANT Delete ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantDelete nvarchar(1000)',@GrantDelete;

	set @UseETL = 'Use [ETL]; exec(@GrantExecute)';
	set @GrantExecute = 'GRANT EXECUTE ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantExecute nvarchar(1000)',@GrantExecute;

	set @UseETL = 'Use [ETL]; exec(@GrantInsert)';
	set @GrantInsert = 'GRANT Insert ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantInsert nvarchar(1000)',@GrantInsert;

	set @UseETL = 'Use [ETL]; exec(@GrantReferences)';
	set @GrantReferences = 'GRANT References ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantReferences nvarchar(1000)',@GrantReferences;

	set @UseETL = 'Use [ETL]; exec(@GrantSelect)';
	set @GrantSelect = 'GRANT Select ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantSelect nvarchar(1000)',@GrantSelect;

	set @UseETL = 'Use [ETL]; exec(@GrantUpdate)';
	set @GrantUpdate = 'GRANT Update ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantUpdate nvarchar(1000)',@GrantUpdate;

	set @UseETL = 'Use [ETL]; exec(@GrantViewDef)';
	set @GrantViewDef = 'GRANT VIEW DEFINITION ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
	exec sp_executesql @UseETL, N'@GrantViewDef nvarchar(1000)',@GrantViewDef;

end





