﻿


CREATE procedure [ETL].[CreateProjectAccess_Original20150212]
@ProjectName nvarchar(20),
@Password nvarchar(20)
as
/*******************************************************************************************
Description:  Performs Steps 5-8 from the Configuring Project ETL Server Access document

			  
Parameters:
	@ProjectName - Name of the project being configured
	@Password - Password for the VHAMASTER\_ProjectName_ETL account
	Both of these values should be in the Kanban card 
	
	
Example: Exec ETL.ETL.CreateProjectAccess 'PBM_Analytics','gT*5$m2ZsP!6!w4'


*******************************************************************************************/



declare				--@ProjectName varchar(20),
					@UseETL nvarchar(50)
					,@CreateSchema nvarchar(100)
					,@CreateDevUser nvarchar(500)
					,@CreateETLUser nvarchar(500)
					,@ChangeOwner nvarchar(500)
					,@GrantAlter nvarchar(500)
					,@GrantDelete nvarchar(500)
					,@GrantExecute nvarchar(500)
					,@GrantInsert nvarchar(500)
					,@GrantReferences nvarchar(500)
					,@GrantSelect nvarchar(500)
					,@GrantUpdate nvarchar(500)
					,@GrantViewDef nvarchar(500)
					,@UseMaster nvarchar(50)
					,@CreateDevLogin nvarchar(1000)
					,@CreateETLLogin nvarchar(1000)
					,@AlterDevRole nvarchar(1000)
					,@UseMSDB nvarchar(50)
					,@CreateProxy nvarchar(1000)
					,@CreateCredential nvarchar(1000)
					--,@Password nvarchar(20)
					,@GrantProxy nvarchar(1000)
					,@CreateMSDBUser nvarchar(1000)
					,@AddRole nvarchar(1000)
					,@UseSSISDB nvarchar(1000)
					,@CreateSSISDBUser nvarchar(1000)
					,@folder_id int
					,@principal_id int

--set @ProjectName = 'PACT_CC'
--set @Password = 'Z!y4$2Pt7p*K$'


--steps performed in the ETL database

--CREATE SCHEMA
set @UseETL = 'Use [ETL]; exec(@CreateSchema)';
set @CreateSchema = 'CREATE SCHEMA ['+@ProjectName+'];'
exec sp_executesql @UseETL, N'@CreateSchema nvarchar(1000)',@CreateSchema;

--CREATE Dev USER
set @UseETL = 'Use [ETL]; exec(@CreateDevUser)';
set @CreateDevUser = 'CREATE USER [u'+@ProjectName+'_Dev] FOR LOGIN [VHAMASTER\'+@ProjectName+'_Dev] WITH DEFAULT_SCHEMA=['+@ProjectName+']'
exec sp_executesql @UseETL, N'@CreateDevUser nvarchar(1000)',@CreateDevUser;

--CREATE ETL USER
set @UseETL = 'Use [ETL]; exec(@CreateETLUser)';
set @CreateETLUser = 'CREATE USER [u'+@ProjectName+'_ETL] FOR LOGIN [VHAMASTER\'+@ProjectName+'_ETL] WITH DEFAULT_SCHEMA=['+@ProjectName+']'
exec sp_executesql @UseETL, N'@CreateETLUser nvarchar(1000)',@CreateETLUser;

--make Dev user owner of schema
set @UseETL = 'Use [ETL]; exec(@ChangeOwner)';
set @ChangeOwner ='ALTER AUTHORIZATION ON SCHEMA::'+@ProjectName+' TO u'+@ProjectName+'_Dev;'
exec sp_executesql @UseETL, N'@ChangeOwner nvarchar(1000)',@ChangeOwner;

--grant permissions on schema
set @UseETL = 'Use [ETL]; exec(@GrantAlter)';
set @GrantAlter = 'GRANT ALTER ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantAlter nvarchar(1000)',@GrantAlter;

set @UseETL = 'Use [ETL]; exec(@GrantDelete)';
set @GrantDelete = 'GRANT Delete ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantDelete nvarchar(1000)',@GrantDelete;

set @UseETL = 'Use [ETL]; exec(@GrantExecute)';
set @GrantExecute = 'GRANT EXECUTE ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantExecute nvarchar(1000)',@GrantExecute;

set @UseETL = 'Use [ETL]; exec(@GrantInsert)';
set @GrantInsert = 'GRANT Insert ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantInsert nvarchar(1000)',@GrantInsert;

set @UseETL = 'Use [ETL]; exec(@GrantReferences)';
set @GrantReferences = 'GRANT References ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantReferences nvarchar(1000)',@GrantReferences;

set @UseETL = 'Use [ETL]; exec(@GrantSelect)';
set @GrantSelect = 'GRANT Select ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantSelect nvarchar(1000)',@GrantSelect;

set @UseETL = 'Use [ETL]; exec(@GrantUpdate)';
set @GrantUpdate = 'GRANT Update ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantUpdate nvarchar(1000)',@GrantUpdate;

set @UseETL = 'Use [ETL]; exec(@GrantViewDef)';
set @GrantViewDef = 'GRANT VIEW DEFINITION ON SCHEMA :: '+@ProjectName+' TO u'+@ProjectName+'_ETL;'
exec sp_executesql @UseETL, N'@GrantViewDef nvarchar(1000)',@GrantViewDef;


--steps performed in the system databases (msdb, master)


--CREATE Dev LOGIN
set @UseMaster = 'Use [master]; exec(@CreateDevLogin)';
set @CreateDevLogin = 'CREATE LOGIN [VHAMASTER\'+@ProjectName+'_Dev] FROM WINDOWS WITH DEFAULT_DATABASE=[ETL], DEFAULT_LANGUAGE=[us_english]'
exec sp_executesql @UseMaster, N'@CreateDevLogin nvarchar(1000)',@CreateDevLogin;


--CREATE ETL LOGIN
set @UseMaster = 'Use [master]; exec(@CreateETLLogin)';
set @CreateETLLogin = 'CREATE LOGIN [VHAMASTER\'+@ProjectName+'_ETL] FROM WINDOWS WITH DEFAULT_DATABASE=[ETL], DEFAULT_LANGUAGE=[us_english]'
exec sp_executesql @UseMaster, N'@CreateETLLogin nvarchar(1000)',@CreateETLLogin;

--add Dev user to sysadmin
set @UseMaster = 'Use [master]; exec(@AlterDevRole)';
set @AlterDevRole = 'ALTER SERVER ROLE [sysadmin] ADD MEMBER [VHAMASTER\'+@ProjectName+'_Dev]'
exec sp_executesql @UseMaster, N'@AlterDevRole nvarchar(1000)',@AlterDevRole;



--create credential
set @CreateCredential = 'CREATE CREDENTIAL '+@ProjectName+'_ETL WITH IDENTITY = '+''''+'VHAMASTER\'+@ProjectName+'_ETL'+''''+', SECRET = '+''''+@Password+''''+';'
exec sp_executesql @CreateCredential

--create proxy
set @UseMSDB = 'Use [msdb]; exec(@CreateProxy)';
set @CreateProxy = 'EXEC msdb.dbo.sp_add_proxy @proxy_name=N'+''''+@ProjectName+'_ETL'+''''+',@credential_name=N'+''''+@ProjectName+'_ETL'+''''+', @enabled=1'
exec sp_executesql @UseMSDB, N'@CreateProxy nvarchar(1000)',@CreateProxy;


set @GrantProxy = 'EXEC msdb.dbo.sp_grant_proxy_to_subsystem @proxy_name=N'+''''+@ProjectName+'_ETL'+''''+', @subsystem_id=3'
exec sp_executesql @GrantProxy
set @GrantProxy = 'EXEC msdb.dbo.sp_grant_proxy_to_subsystem @proxy_name=N'+''''+@ProjectName+'_ETL'+''''+', @subsystem_id=9'
exec sp_executesql @GrantProxy
set @GrantProxy = 'EXEC msdb.dbo.sp_grant_proxy_to_subsystem @proxy_name=N'+''''+@ProjectName+'_ETL'+''''+', @subsystem_id=10'
exec sp_executesql @GrantProxy
set @GrantProxy = 'EXEC msdb.dbo.sp_grant_proxy_to_subsystem @proxy_name=N'+''''+@ProjectName+'_ETL'+''''+', @subsystem_id=11'
exec sp_executesql @GrantProxy
set @GrantProxy = 'EXEC msdb.dbo.sp_grant_login_to_proxy @proxy_name=N'+''''+@ProjectName+'_ETL'+''''+', @login_name=N'+''''+'VHAMASTER\'+@ProjectName+'_ETL'+''''
exec sp_executesql @GrantProxy

--CREATE MSDB USER
set @UseMSDB = 'Use [msdb]; exec(@CreateMSDBUser)';
set @CreateMSDBUser = 'CREATE USER [u'+@ProjectName+'_ETL] FOR LOGIN [VHAMASTER\'+@ProjectName+'_ETL] WITH DEFAULT_SCHEMA=[dbo]'
exec sp_executesql @UseMSDB, N'@CreateMSDBUser nvarchar(1000)',@CreateMSDBUser;

--add roles to user
set @UseMSDB = 'Use [msdb]; exec(@AddRole)';
set @AddRole = 'EXEC sp_addrolemember '+''''+'SQLAgentOperatorRole'+''''+','+''''+'u'+@ProjectName+'_ETL'+''''
exec sp_executesql @UseMSDB, N'@AddRole nvarchar(1000)',@AddRole;


--Add SSISDB folder
EXEC SSISDB.[catalog].[create_folder] @ProjectName


--CREATE SSIDDB USER
set @UseSSISDB = 'Use [SSISDB]; exec(@CreateSSISDBUser)';
set @CreateSSISDBUser = 'CREATE USER [u'+@ProjectName+'_ETL] FOR LOGIN [VHAMASTER\'+@ProjectName+'_ETL] WITH DEFAULT_SCHEMA=[catalog]'
exec sp_executesql @UseSSISDB, N'@CreateSSISDBUser nvarchar(1000)',@CreateSSISDBUser;

--grant permissions to folder
set @folder_id = (select folder_id from SSISDB.[internal].[folders] where [name]=@ProjectName)
set @principal_id = (select principal_id from SSISDB.sys.database_principals where [name]='u'+@ProjectName+'_ETL')


exec SSISDB.[catalog].[grant_permission] 1,@folder_id,@principal_id,1
exec SSISDB.[catalog].[grant_permission] 1,@folder_id,@principal_id,101
exec SSISDB.[catalog].[grant_permission] 1,@folder_id,@principal_id,103





