﻿
--
--
--
CREATE PROCEDURE [RTLS_NDRDW].[JobCreate] 
	@JobName NVARCHAR(1024)
	, @JobOrder INT
	, @ConnectionMasterID INT
	, @ScheduleMasterID INT
	, @SystemMasterID INT
	, @FlightMasterID INT

AS
BEGIN
DECLARE @stmt AS NVARCHAR(MAX)
DECLARE @schema AS NVARCHAR(256)
DECLARE @job_id AS INT
--
--
--

SET @schema = 'RTLS_NDRDW'


SET @stmt='
	BEGIN TRANSACTION

	DECLARE @JobMasterID AS BIGINT
	DECLARE @SystemMasterID AS INT
	DECLARE @FlightMasterID AS INT
	DECLARE @id AS BIGINT
	DECLARE @dt AS DATETIME

	SET @dt = GETDATE()
	SET @SystemMasterID = ' + CAST(@SystemMasterID AS NVARCHAR(5) ) + '
	SET @FlightMasterID = ' + CAST(@FlightMasterID AS NVARCHAR(5) ) + '

	-- job master
	INSERT INTO [' + @schema + '].[JobMaster]
		([JobMasterName]
		,[JobMasterOrder]
		,[IsEnabled]
		,[JobMasterTimeZone]
		,[JobMasterStartTime]
		,[ConnectionMasterID]
		,[ScheduleMasterID]
		,[CreateDate]
		,[UpdateDate])
     VALUES
	 ( ''' + @JobName + '''
	 , ' + CAST( @JobOrder AS NVARCHAR(5) ) + '
	 , 1
	 , ''CST''
	 , ''00:00:00''
	 , ' + CAST( @ConnectionMasterID AS NVARCHAR(5) ) + '
	 , ' + CAST( @ScheduleMasterID AS NVARCHAR(5) ) + '
	 , @dt
	 , @dt
	 )

	SELECT @JobMasterID = @@IDENTITY

	-- run master
	INSERT INTO [' + @schema + '].[RunMaster]
		([JobMasterID]
		,[FlightMasterID]
		,[TableMasterID]
		,[RunMasterName]
		,[LoggingLevelOverride]
		,[PrecedentRun]
		,[RunOrderSeq]
		,[IsEnabled]
		,[CreateDate]
		,[UpdateDate])

	SELECT 
		@JobMasterID AS [JobMasterID]
		, @FlightMasterID AS [FlightMasterID]
		, [b].[TableMasterID] AS [TableMasterID]
		, ( [a].[SystemName] + ''.'' + [b].[TableName] ) AS [RunMasterName]
		, 1 AS [LoggingLevelOverride]
		, 0 AS [PrecedentRun]
		, ROW_NUMBER() OVER ( PARTITION BY [a].[SystemMasterID] ORDER BY [a].[SystemMasterID] ) AS [RunOrderSeq]
		, 1 AS [IsEnabled]
		, @dt AS [CreateDate]
		, @dt AS [UpdateDate]
	FROM [RTLS_NDRDW].[SystemMaster] [a]
	JOIN [RTLS_NDRDW].[TableMaster] [b] ON [a].[SystemMasterID] = [b].[SystemMasterID]
	WHERE [a].[SystemMasterID] = @SystemMasterID
	AND [b].[IsEnabled] = 1
	ORDER BY
		[a].[SystemMasterID]
		, [b].[TableMasterID]

	COMMIT TRANSACTION
	PRINT ''JobMasterID: '' + CAST( @JobMasterID AS NVARCHAR(25))
	'


	-- PRINT @stmt
	PRINT 'Job Created: ' + @JobName + ' @SystemMasterID=' + CAST( @SystemMasterID AS NVARCHAR(5) ) + ' @FlightMasterID=' + CAST( @FlightMasterID AS NVARCHAR(5) )
	EXEC sp_executesql @stmt=@stmt
END
