﻿-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/08/2015
-- Description:	This procedure will correlate
--				and store database disk space 
--				usage
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptCaptureDBDiskSpaceHistory] 
AS
BEGIN

	SET NOCOUNT ON;

------------------------------Data file size----------------------------
--if exists (select * from tempdb.sys.all_objects where name like '%#dbsize%')
--drop table #dbsize
create table #dbsize
	(dbname	sysname,
	DID int,
	fileid int,
			dbstatus varchar(50),
			Recovery_Model varchar(40) default ('NA'), 
			file_Size_MB decimal(30,2)default (0),
			Space_Used_MB decimal(30,2)default (0),
			Free_Space_MB decimal(30,2) default (0))

 
insert into #dbsize(Dbname,did, fileid, dbstatus,Recovery_Model,file_Size_MB,Space_Used_MB,Free_Space_MB)
exec sp_msforeachdb
'use [?];
	select	DB_NAME() AS DbName
			,DB_ID() AS DID
			,file_id as fileid
			,CONVERT(varchar(20),DatabasePropertyEx(''?'',''Status''))
			,CONVERT(varchar(20),DatabasePropertyEx(''?'',''Recovery''))
			,sum(size)/128.0 AS File_Size_MB
			,sum(CAST(FILEPROPERTY(name, ''SpaceUsed'') AS INT))/128.0 as Space_Used_MB
			,SUM( size)/128.0 - sum(CAST(FILEPROPERTY(name,''SpaceUsed'') AS INT))/128.0 AS Free_Space_MB 
from sys.database_files  --CROSS APPLY sys.dm_os_volume_stats(DB_ID(sys.database_files.name), sys.database_files.file_id)
where type=0 group by type, file_id'
 
-------------------log size--------------------------------------
--if exists (select * from tempdb.sys.all_objects where name like '#logsize%')
--	drop table #logsize
create table #logsize
	(Dbname sysname,
	did int,
	fileid int, 
	Log_File_Size_MB decimal(38,2)default (0),
	log_Space_Used_MB decimal(30,2)default (0),
	log_Free_Space_MB decimal(30,2)default (0))

insert into #logsize(Dbname, did, fileid, Log_File_Size_MB,log_Space_Used_MB,log_Free_Space_MB)
exec sp_msforeachdb
'use [?];
	select	DB_NAME() AS DbName
			,DB_ID() AS did
			,file_id as fileid
			,sum(size)/128.0 AS Log_File_Size_MB
			,sum(CAST(FILEPROPERTY(name, ''SpaceUsed'') AS INT))/128.0 as log_Space_Used_MB
			,SUM( size)/128.0 - sum(CAST(FILEPROPERTY(name,''SpaceUsed'') AS INT))/128.0 AS log_Free_Space_MB 
from sys.database_files  
where type=1 group by type, file_id'
 
 

--------------------------------database free size
--if exists (select * from tempdb.sys.all_objects where name like '%#dbfreesize%')
--	drop table #dbfreesize
create table #dbfreesize
	(name sysname,
	did int,
	fileid int,
	database_size varchar(50),
	Freespace varchar(50)default (0.00))
 
insert into #dbfreesize(name, database_size,Freespace)
exec sp_msforeachdb
'use [?];
	SELECT	database_name = db_name()
			,database_size = ltrim(str((convert(DECIMAL(15, 2), dbsize) + convert(DECIMAL(15, 2), logsize)) * 8192 / 1048576, 15, 2) + ''MB'')
			,''unallocated space'' = ltrim(str((
                CASE 
                    WHEN dbsize >= reservedpages
                        THEN (convert(DECIMAL(15, 2), dbsize) - convert(DECIMAL(15, 2), reservedpages)) * 8192 / 1048576
                    ELSE 0
                    END
                ), 15, 2) + '' MB'')
	FROM (SELECT dbsize = sum(convert(BIGINT, CASE 
                    WHEN type = 0
                        THEN size
                    ELSE 0
                    END))
			,logsize = sum(convert(BIGINT, CASE 
                    WHEN type <> 0
                        THEN size
                    ELSE 0
                    END))
			FROM sys.database_files
	) AS files
,(
    SELECT reservedpages = sum(a.total_pages)
        ,usedpages = sum(a.used_pages)
        ,pages = sum(CASE 
                WHEN it.internal_type IN (
                        202
                        ,204
                        ,211
                        ,212
                        ,213
                        ,214
                        ,215
                        ,216
                        )
                    THEN 0
                WHEN a.type <> 1
                    THEN a.used_pages
                WHEN p.index_id < 2
                    THEN a.data_pages
                ELSE 0
                END)
    FROM sys.partitions p
    INNER JOIN sys.allocation_units a
        ON p.partition_id = a.container_id
    LEFT JOIN sys.internal_tables it
        ON p.object_id = it.object_id
) AS partitions'
-----------------------------------
 
 
 
--if exists (select * from tempdb.sys.all_objects where name like '%#alldbstate%')
--	drop table #alldbstate 
create table #alldbstate 
	(dbname sysname,
	 did int,
	 fileid int,
	DBstatus varchar(55),
	R_model Varchar(30))
  
--select * from sys.master_files
 
insert into #alldbstate (dbname,DBstatus,R_model)
select name,CONVERT(varchar(20),DATABASEPROPERTYEX(name,'status')),recovery_model_desc from sys.databases
--select * from #dbsize
 
insert into #dbsize(Dbname,dbstatus,Recovery_Model)
select dbname,dbstatus,R_model from #alldbstate where DBstatus <> 'online'
 
insert into #logsize(Dbname)
select dbname from #alldbstate where DBstatus <> 'online'
 
insert into #dbfreesize(name)
select dbname from #alldbstate where DBstatus <> 'online'

insert into [RTLS_NDRDW].[rptDatabaseDiskSpaceHistory] 
select	d.Dbname
		,d.did
		,d.fileid
		,d.dbstatus
		,d.Recovery_Model,
		(file_size_mb + log_file_size_mb) as DBsize,
		d.file_Size_MB
		,d.Space_Used_MB
		,d.Free_Space_MB,
		l.Log_File_Size_MB
		,log_Space_Used_MB
		,l.log_Free_Space_MB
		,fs.Freespace as DB_Freespace
		,(d.space_used_mb) / (total_bytes/1024/1024) * 100 as PercDiskUsed
		,(total_bytes/1024/1024) as TotalDiskSpaceMB
		,(available_bytes/1024/1024) as AvailDiskSpaceMB
		,getdate() as PullDate
--into [ETLDashboard].[dbo].[rptDatabaseDiskSpace]
from #dbsize d join #logsize l on d.Dbname=l.Dbname 
	join #dbfreesize fs on d.Dbname=fs.name
	CROSS APPLY sys.dm_os_volume_stats(d.did, d.fileid)
--group by d.dbname, d.did, d.fileid, d.dbstatus, d.recovery_model, d.file_size_mb, d.free_space_mb, l.log_file_size_mb, log_space_used_mb
--	,l.log_free_space_mb, fs.freespace
order by Dbname

END

