﻿

-- =============================================
-- Author:		Michael D. Hanson
--				HP Enterprise Services
-- Create date: 04-15-2015
-- Description:	This procedure provides job 
--				counts for the ETL Framework
--				Dashboard
-- Parameters:	@StartDate - beginning date range
--				@EndDate - ending date range
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptDashBoardCounts] --'05/16/2015', '05/22/2015'
@StartDate as DateTime,
@EndDate as DateTime
as 
BEGIN


BEGIN
	WITH e AS 
	(SELECT [a].[ExecutionLogStatusID] as [status], 
			[b].[JobMasterName] as [folder_name], 
			COUNT([a].[ExecutionLogStatusID]) AS StatCount
	FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
		LEFT JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
		JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
	WHERE [c].[ExecutionJobLogStart] BETWEEN @StartDate AND @EndDate 
	GROUP BY [a].[ExecutionLogStatusID], [b].[JobMasterName]
	),
	p AS
	(SELECT [b].[JobMasterName] AS pkgPath
	FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
		LEFT JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
		JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
	WHERE (CONVERT(DATETIME, [c].[ExecutionJobLogStart]) BETWEEN @StartDate AND @EndDate)
	)
	    
	SELECT 'dd' AS DrillDown, 
			ISNULL([folder_name], 'NA') as [folder_name], 
			ISNULL(MAX(CASE [status] WHEN 3 THEN StatCount ELSE 0 END), 0) AS 'Failed', 
			ISNULL(MAX(CASE [status] WHEN 1 THEN StatCount ELSE 0 END), 0) AS 'Running', 
			ISNULL(MAX(CASE [status] WHEN 2 THEN StatCount ELSE 0 END), 0) AS 'Success', 
			ISNULL(MAX(CASE [status] WHEN 0 THEN StatCount ELSE 0 END), 0) AS 'Other', 
			ISNULL(SUM(StatCount), 0) AS StatCount,
			(select COUNT(DISTINCT([pkgPath])) from p) as nDistinctPackages,
			(SELECT COUNT(1) AS Expr1 FROM [RTLS_NDRDW].[JobMaster]) AS nAllPackages
	FROM e AS e_1
	GROUP BY folder_name
	ORDER BY folder_name
END

END
