﻿

-- =============================================
-- Author:		Michael D. Hanson
--				HP Enterprise Services
-- Create date: 04-15-2015
-- Description:	This procedure provides job 
--				execution data for the ETL
--				Dashboard
-- Parameters:	@StartDate - beginning date range
--				@EndDate - ending date range
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptDashBoardExecution] --'05/16/2015','05/18/2015', 3
@StartDate as DateTime,
@EndDate as DateTime,
@StatusCode as int
as
BEGIN

DECLARE @xStartDate DateTime, @xEndDate as DateTime, @xStatusCode as int
SET @xStartDate = @StartDate
SET @xEndDate = @EndDate
SET @xStatusCode = @StatusCode

	SELECT	 [EL].[ExecutionLogStatusID]
			,[EL].[execution_id]
			,[PM].[PackageFolder]
			,[PM].[PackageProject]
			,[msg].[Package_Name] as [PackageName]
			,[EL].[ExecutionLogStart]
			,[EL].[ExecutionLogStop]
			,DATEDIFF( SECOND, [EJL].[ExecutionJobLogStart], [EJL].[ExecutionJobLogStop] ) AS Duration
			,[EL].[JobMasterID]
			,[EL].[RunMasterID]
			,[JM].[JobMasterName]
			,[FM].[FlightName]
			,[VM].[VISNName]
			,[VM].[VISNLongName]
			,CASE WHEN LEN(op.[message]) <= 4096 THEN op.[message] ELSE LEFT(op.[message], 1024) + '...' END AS [message]
	FROM [RTLS_NDRDW].[ExecutionLog] [EL] with (nolock)
		   INNER JOIN [RTLS_NDRDW].[VISNMaster] [VM] with (nolock) on [EL].[VISNMasterID] = [VM].[VISNMasterID]
		   INNER JOIN [RTLS_NDRDW].[JobMaster] [JM] with (nolock) on [EL].[JobMasterID] = [JM].[JobMasterID]
		   INNER JOIN [RTLS_NDRDW].[FlightMaster] [FM] with (nolock) on [EL].[FlightMasterID] = [FM].[FlightMasterID]
		   INNER JOIN [RTLS_NDRDW].[PackageMaster] [PM] with (nolock) on [EL].[PackageMasterID] = [PM].[PackageMasterID]
		   INNER JOIN [RTLS_NDRDW].[ExecutionJobLog] [EJL] with (nolock) on [EL].[ExecutionJobLogID] = [EJL].[ExecutionJobLogID]
		   INNER JOIN [ssisdb].[internal].[event_messages] msg ON msg.[operation_id] = [EL].[execution_id]
		   INNER JOIN [SSISDB].[internal].[operation_messages] op ON op.[operation_message_id] = msg.[event_message_id]
		   --INNER JOIN [RTLS_NDRDW].[ExecutionCode] [EC] on [EL].[ExecutionLogStatusID] = [EC].[CodeID]
	WHERE	[EL].[ExecutionLogStatusID] = @xStatusCode
			AND [EJL].[ExecutionJobLogStart] BETWEEN @xStartDate and @xEndDate
			AND op.[message_type] = 120

END


