﻿-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/22/2015
-- Update date: 03/03/2015
--				Updated to utilize ETL Framework
--				Metadata Discovery Data
--
--				03/13/2015
--				Updated to add mulitple flags
--				to show different views
--				added columns to help 
--				differentiate old and new 
--				values
--
--				05/21/2015
--				Updated to reflect new Key
--				Column location 
--
-- Description:	Pull MetaData values for the
--				Data Dictionary report 
--				based on database namve passed
--				in
-- Parameters:	
--				@Database: name of dB to pull
--					values from
--				@Flag: type of report to run
--					1: Current Version 
--					2: Base Version (original values)
--					3: Base + changes (changes to the base shown)
--					4: Changes only (only show changes)
-- =============================================

CREATE PROCEDURE [RTLS_NDRDW].[rptEDAReportView] 'insites', 3
	-- Add the parameters for the stored procedure here
	@xDatabase varchar(128) = 'insites',
	@xFlag int = 1
AS


DECLARE @Database varchar(128), @Flag int;

SET @Database = @xDatabase
SET @Flag = @xFlag;

BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	DECLARE @BASE date
	SET @BASE = (SELECT CONVERT(DATE, MIN(CreateDate)) FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges]);
	
	IF @Flag = 1 --CURENT VERSION OF REPORT
		BEGIN
			SELECT	sm.SystemName as DatabaseName
				,sm.Description as DatabaseDescription
				,md.TableName
				,md.TableOwner as TableSchema
				,md.ObjectDescription as TableDescription
				/*,CASE
					WHEN ky.ColumnName is not NULL then ky.ColumnName
					ELSE md.ColumnName
				 END as ColumnName
				 */
				,md.ColumnName
				,md.CurrentMetadataExtractID as ColumnOrder
				, CASE
					WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
					ELSE ISNULL(md.ObjectDescription, '')
				  END AS ColumnDescription
				, '' as TModID
				, '' as TModTime
				, '' as CModID
				, '' as CModTime
				, '' as DMLID
				, md.DataType
				, md.ColumnLength as [Length]
				, md.DefaultValue
				, md.KeyType as [Key]
				, md.AllowsNulls as NULLSAllowed
				--, ky.ConstraintName as KeyName
				--, ky.ConstraintType as KeyType
			FROM	[RTLS_NDRDW].[CurrentMetadataExtract] md WITH (NOLOCK)
					JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
					--LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
						--and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
			WHERE sm.SystemName = @Database 
				  and md.ObjectType is NULL
		    OPTION (OPTIMIZE FOR UNKNOWN);
		END 
	ELSE IF @Flag = 2 --BASE VERSION
		BEGIN
			SELECT	sm.SystemName as DatabaseName
					,sm.Description as DatabaseDescription
					,md.TableName
					,md.TableOwner as TableSchema
					,md.ObjectDescription as TableDescription
					/*,CASE
						WHEN ky.ColumnName is not NULL then ky.ColumnName
						ELSE md.ColumnName
					 END as ColumnName
					 */
					,md.ColumnName
					,md.CurrentMetadataExtractID as ColumnOrder
					, CASE
						WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
						ELSE ISNULL(md.ObjectDescription, '')
						END AS ColumnDescription
					, '' as TModID
					, '' as TModTime
					, '' as CModID
					, '' as CModTime
					, '' as DMLID
					, md.DataType
					, md.ColumnLength as [Length]
					, md.DefaultValue
					, md.KeyType as [Key]
					, md.AllowsNulls as NULLSAllowed
					--, ky.ConstraintName as KeyName
					--, ky.ConstraintType as KeyType
			FROM	[RTLS_NDRDW].[ETLMetadataIncrementalChanges] md WITH (NOLOCK)
					JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
					--LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
					--	and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
			WHERE sm.SystemName = @Database 
					and md.IsPulledByMetaTool = 1
					and (md.ObjectType is NULL)
					and CONVERT(DATE, md.CreateDate) = (@BASE)
			OPTION (OPTIMIZE FOR UNKNOWN);
		END
	ELSE IF @Flag = 3 --BASE VERSION WITH CHANGES
	BEGIN
			WITH ColumnModification as (
				SELECT	TableName,
						ColumnName,
						CONVERT(DATE, CreateDate) as ModTime,
						SUM([DMLOperationID]) as ModSUM,
						CASE 
							WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
							WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
							WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
							ELSE ''
						END as ModID
				FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
				WHERE ([IsPulledByMetaTool] = 0) 
					  --and CONVERT(Date, CreateDate) in (@Date, @BASE)
					  and ObjectType is NULL
				GROUP BY TableName, ColumnName ,CONVERT(DATE, CreateDate)
			), TableModification as (
					SELECT	TableName,
						CONVERT(DATE, CreateDate) as ModTime,
						SUM([DMLOperationID]) as ModSUM,
						CASE 
							WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
							WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
							WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
							ELSE ''
						END as ModID
				FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
				WHERE ([IsPulledByMetaTool] = 0) 
					  --and CONVERT(Date, CreateDate) = @Date
					  and ObjectType = 'U'
				GROUP BY TableName, CONVERT(Date, CreateDate)
			)
			SELECT	sm.SystemName as DatabaseName
					,sm.Description as DatabaseDescription
					,md.TableName
					,md.TableOwner as TableSchema
					,md.ObjectDescription as TableDescription
					/*,CASE
						WHEN ky.ColumnName is not NULL then ky.ColumnName
						ELSE md.ColumnName
					 END as ColumnName
					 */
					,md.ColumnName
					,md.CurrentMetadataExtractID as ColumnOrder
					, CASE
						WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
						ELSE ISNULL(md.ObjectDescription, '')
					  END AS ColumnDescription
					, t.ModID as TModID
					, t.ModTime as TModTime
					, m.ModID as CModID
					, m.ModTime as CModTime
					, CASE 
						WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 2 and (CONVERT(DATE, md.CreateDate) > @Base))  THEN 'NEW' 
						WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 3 and (CONVERT(DATE, md.CreateDate) > @Base)) THEN 'OLD'
						WHEN (m.ModID is null) THEN ''
						ELSE 'OLD'
					  END as DMLID
					, md.DataType
					, md.ColumnLength as [Length]
					, md.DefaultValue
					, md.KeyType as [Key]
					, md.AllowsNulls as NULLSAllowed
					--, ky.ConstraintName as KeyName
					--, ky.ConstraintType as KeyType
			FROM	[RTLS_NDRDW].[ETLMetadataIncrementalChanges] md WITH (NOLOCK)
					JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
					--LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
					--	and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
					LEFT JOIN ColumnModification m WITH (NOLOCK) on md.[TableName] = m.[TableName] and md.[ColumnName] = m.[ColumnName]
					LEFT JOIN TableModification t WITH (NOLOCK) on md.[TableName] = t.[TableName]
			WHERE sm.SystemName = @Database 
				  --and CONVERT(DATE, md.CreateDate) = @Date
				  and (md.ObjectType is NULL)
		    OPTION (OPTIMIZE FOR UNKNOWN);
	END
	ELSE
	BEGIN --CHANGES ONLY
		WITH ColumnModification as (
			SELECT	TableName,
					ColumnName,
					CONVERT(DATE, CreateDate) as ModTime,
					SUM([DMLOperationID]) as ModSUM,
					CASE 
						WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
						WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
						WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
						ELSE ''
					END as ModID
			FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
			WHERE ([IsPulledByMetaTool] = 0) 
					--and CONVERT(Date, CreateDate) in (@Date, @BASE)
					and ObjectType is NULL
			GROUP BY TableName, ColumnName ,CONVERT(DATE, CreateDate)
		), TableModification as (
				SELECT	TableName,
					CONVERT(DATE, CreateDate) as ModTime,
					SUM([DMLOperationID]) as ModSUM,
					CASE 
						WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
						WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
						WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
						ELSE ''
					END as ModID
			FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
			WHERE ([IsPulledByMetaTool] = 0) 
					--and CONVERT(Date, CreateDate) = @Date
					and ObjectType = 'U'
			GROUP BY TableName, CONVERT(Date, CreateDate)
		)
		SELECT	sm.SystemName as DatabaseName
				,sm.Description as DatabaseDescription
				,md.TableName
				,md.TableOwner as TableSchema
				,md.ObjectDescription as TableDescription
				/*,CASE
					WHEN ky.ColumnName is not NULL then ky.ColumnName
					ELSE md.ColumnName
				 END as ColumnName
				 */
				,md.ColumnName
				,md.CurrentMetadataExtractID as ColumnOrder
				, CASE
					WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
					ELSE ISNULL(md.ObjectDescription, '')
					END AS ColumnDescription
				, t.ModID as TModID
				, t.ModTime as TModTime
				, m.ModID as CModID
				, m.ModTime as CModTime
				, CASE 
					WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 2 and (CONVERT(DATE, md.CreateDate) > @Base))  THEN 'NEW' 
					WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 3 and (CONVERT(DATE, md.CreateDate) > @Base)) THEN 'OLD'
					WHEN (m.ModID is null) THEN ''
					ELSE ''
				  END as DMLID
				, md.DataType
				, md.ColumnLength as [Length]
				, md.DefaultValue
				, md.KeyType as [Key]
				, md.AllowsNulls as NULLSAllowed
				--, ky.ConstraintName as KeyName
				--, ky.ConstraintType as KeyType
		FROM	[RTLS_NDRDW].[ETLMetadataIncrementalChanges] md WITH (NOLOCK)
				JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
				--LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
				--	and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
				LEFT JOIN ColumnModification m WITH (NOLOCK) on md.[TableName] = m.[TableName] and md.[ColumnName] = m.[ColumnName]
				LEFT JOIN TableModification t WITH (NOLOCK) on md.[TableName] = t.[TableName]
		WHERE sm.SystemName = @Database 
				and md.IsPulledByMetaTool = 0
				and (md.ObjectType is NULL)
		OPTION (OPTIMIZE FOR UNKNOWN);
	END


END