﻿

-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04/15/2015
-- Description:	Gets ETL Job execution stats 
--				from ETL framework data
--				for ETL Dashboard Report
-- Parameters:	@StartDate: start date of data pull
--				@EndDate: end date of data pull
-- =============================================

CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionStatistics] --'04/01/2015', '04/16/2016'
@StartDate as DateTime,
@EndDate as DateTime

AS
BEGIN

WITH Stat as 
(
SELECT DISTINCT
	  [a].[ExecutionJobLogID]
	, DATEDIFF( SECOND, [c].[ExecutionJobLogStart], [c].[ExecutionJobLogStop]) AS [ExecutionJobDurationSeconds]
	, SUM([a].[ExecutionLogRowsProcessed]) as RowsProcessed
	, SUM([a].[ExecutionLogBytesTransferred]) as BytesTransferred
	, COUNT( [a].[ExecutionLogID] ) AS [TotalCount]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 0 THEN 1 ELSE 0 END ) AS [Pending]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 2 THEN 1 ELSE 0 END ) AS [TotalSuccess]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 3 THEN 1 ELSE 0 END ) AS [TotalError]
	, SUM( CASE WHEN CAST([c].[ExecutionJobLogStop] AS TIME) > CAST([s].[ScheduleStopTime] AS TIME) THEN 1 ELSE 0 END) AS [JobsOutsideWindow]
FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
	LEFT JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
	JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
	LEFT JOIN [RTLS_NDRDW].[ScheduleMaster] [s] on [b].[ScheduleMasterID] = [s].[ScheduleMasterID]
WHERE [c].[ExecutionJobLogStart] BETWEEN @StartDate AND @EndDate
group by [a].[ExecutionJobLogID], [c].[ExecutionJobLogStart], [c].[ExecutionJobLogStop]
)

SELECT
	  AVG([ExecutionJobDurationSeconds]) as AvgTime
	, MIN([ExecutionJobDurationSeconds]) as MinTime
	, MAX([ExecutionJobDurationSeconds]) as MaxTime
	, SUM(RowsProcessed) as RowsProcessed
	, SUM(BytesTransferred) as BytesTransferred
	, SUM([TotalCount]) as TotalCount
	, SUM([JobsOutsideWindow]) as JobsOutsideWindow
	, IIF(SUM(BytesTransferred) = 0, 0, (SUM(BytesTransferred) / CAST(AVG([ExecutionJobDurationSeconds]) AS FLOAT))) as AvgBytesPerSec 
	, SUM([Pending]) as JobsPending
	, (SUM(CAST([TotalSuccess] AS FLOAT)) / SUM(CAST([TotalError] AS FLOAT))) *.1 as SuccessRatio
FROM Stat


END

