﻿

-- =============================================
-- Author:		Michael D. Hanson
--				HP Enterprise Services
-- Create date: 04-15-2015
-- Description:	This procedure calculates 30 day 
--				job counts for the ETL Dashboard
-- Parameters:	none
-- =============================================

CREATE Procedure [RTLS_NDRDW].[rptJobCount] 
AS
BEGIN


WITH RunCount AS
(
SELECT	DISTINCT [a].[ExecutionJobLogID] as [RunExecutionID]
		, CASE [a].[ExecutionLogStatusID]
			WHEN 0 THEN 'Pending'
			WHEN 1 THEN 'Running'
			WHEN 2 THEN 'Success'
			WHEN 3 THEN 'Error'
		END as [RunStatusName]
		,COUNT([a].[ExecutionJobLogID]) as [RunCount]
		,Convert(Date, [c].[ExecutionJobLogStart]) as RunStartDateTime
FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
		LEFT JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
		JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
WHERE (Convert(DateTime, [c].[ExecutionJobLogStart]) BETWEEN Getdate()-30 AND GetDate()) 
GROUP BY [a].[ExecutionJobLogID], [a].[ExecutionLogStatusID], Convert(Date, [c].[ExecutionJobLogStart])
),  DateRanges AS
(
SELECT Convert(Date, Getdate()-30) AS DateValue
UNION ALL
SELECT DATEADD(DAY, 1, DateValue)
FROM DateRanges
WHERE DateValue < Convert(Date, GetDate())
)
SELECT	d.DateValue
		,r.[RunExecutionID]
		,r.[RunStatusName]	
		,ISNULL(SUM(r.[RunCount]), 0) as runs
from [RunCount] r right outer join [DateRanges] d on r.RunStartDateTime = d.DateValue
group by d.DateValue, r.[RunExecutionID], r.[RunStatusName]
order by d.[DateValue], r.[RunExecutionID]

END
