﻿

-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptOverviewExecutionInfo] --'62502'
	-- Add the parameters for the stored procedure here
	@ovrExecutionID int 

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	WITH ref AS
	(
		SELECT
			[execution_id]
			,[reference_id]
			,[reference_type]
			,[environment_folder_name]
			,[environment_name]
		FROM
			[ssisdb].[internal].[executions]
	UNION ALL
		SELECT
			v.[validation_id] AS execution_id
			,r.[reference_id]
			,r.[reference_type]
			,r.[environment_folder_name]
			,r.[environment_name]
		FROM
			[ssisdb].[internal].[environment_references] r RIGHT JOIN [ssisdb].[internal].[validations] v ON r.[reference_id] = v.[reference_id]
	),

	ve AS
	(SELECT [validation_id] AS execution_id, 
			[use32bitruntime], 
			[start_time], 
			[end_time], 
			[caller_name], 
			[folder_name], 
			[project_name], 
			[object_name] as package_name, 
			ENUM.[Description] as [status], 
			'v' AS op_type 
	FROM [ssisdb].[internal].[validations] v 
		INNER JOIN [ssisdb].[internal].[operations] o ON v.[validation_id] = o.[operation_id]
		INNER JOIN [RTLS_NDRDW].[ssisOperationStatus] ENUM on o.[status] = ENUM.[Value]
	WHERE [object_type] = 30
	UNION ALL
	SELECT [execution_id], 
		[use32bitruntime], 
		[start_time], 
		[end_time], 
		[caller_name], 
		[folder_name], 
		[project_name], 
		[package_name], 
		ENUM.[Description] as [status], 
		'e' AS op_type 
	FROM [ssisdb].[internal].[executions] e 
	INNER JOIN [ssisdb].[internal].[operations] o ON e.[execution_id]= o.[operation_id]
	INNER JOIN  [RTLS_NDRDW].[ssisOperationStatus] ENUM on o.[status] = ENUM.[Value]
	)

	SELECT	[execution_id]
			,[folder_name]
			,[project_name]
			,[package_name]
			,[use32bitruntime]
			,[status]
			,CONVERT (datetime, [start_time]) AS start_time
			,CONVERT (datetime, [end_time]) AS end_time
			,CONVERT(FLOAT, DATEDIFF(millisecond, [start_time], ISNULL([end_time], SYSDATETIMEOFFSET())))/1000 AS duration
			,[caller_name]
			,[op_type]
			, (SELECT CASE WHEN [reference_id] IS NULL 
				THEN '-' ELSE (CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' 
				THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name] 
				END  AS env FROM [ref] WHERE [execution_id] = @ovrExecutionID) as env
	FROM [ve]
	WHERE
	[execution_id] = @ovrExecutionID

END

