﻿-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:	@ovrExecutionID: Execution ID
--				used to pull information from
--				SSISDB Catalog
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptOverviewExecutionOverview] --'62520'
	-- Add the parameters for the stored procedure here
	@ovrExecutionID int 

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	SELECT 	executable_statistics.execution_id,
			executables.executable_id, 
		CASE 
			WHEN LEN(executables.[executable_name]) <= 1024 
			THEN executables.[executable_name] 
			ELSE LEFT(executables.[executable_name], 1024) + '...' 
		END AS executable_name, 
		executables.executable_guid, 
                         
		CASE 
			WHEN LEN(executable_statistics.[execution_path]) <= 1024 
			THEN executable_statistics.[execution_path] 
			ELSE LEFT(executable_statistics.[execution_path], 1024) + '...' 
		END AS execution_path, 
		executables.package_name, 
		executable_statistics.statistics_id, 
		CONVERT(FLOAT, executable_statistics.execution_duration) / 1000 AS execution_duration, 
		executable_statistics.execution_result, 
		executable_statistics.execution_value
	FROM	[SSISDB].[internal].[executable_statistics] AS executable_statistics 
		INNER JOIN [SSISDB].[internal].[executables] AS executables 
		ON --executable_statistics.execution_id = executables.execution_id 
		--AND 
		executable_statistics.executable_id = executables.executable_id
	WHERE	(executable_statistics.execution_id = @ovrExecutionID) 
		/*AND (ISNULL(executables.package_name, '') 
			LIKE '%' + REPLACE(REPLACE(REPLACE(@ovrPackageNameContains COLLATE database_default, '[', '[[]'), '_', '[_]'), '%', '[%]') + '%' COLLATE database_default) 
		AND (ISNULL(executable_statistics.execution_path, N'') 
			LIKE '%' + REPLACE(REPLACE(REPLACE(@ovrExecutionPathContains COLLATE database_default, '[', '[[]'), '_', '[_]'), '%', '[%]') + '%' COLLATE database_default) 
		AND (ISNULL(executables.executable_name, N'') 
					LIKE '%' + REPLACE(REPLACE(REPLACE(@ovrExecutableNameContains COLLATE database_default, '[', '[[]'), '_', '[_]'), '%', '[%]') + '%' COLLATE database_default) 
		AND (executable_statistics.execution_result = (CASE WHEN (@ovrResult > - 1) THEN @ovrResult ELSE executable_statistics.[execution_result] END))*/
	ORDER BY executables.executable_id

END

