﻿--
--
--
IF EXISTS (SELECT * FROM sys.objects [a] WHERE type = 'P' AND name = 'ExecutionRunStatusReport' AND [a].[schema_id] = SCHEMA_ID('ETLFramework'))
BEGIN
	DROP PROCEDURE [ETLFramework].[ExecutionRunStatusReport]
	PRINT 'DROP PROCEDURE [ETLFramework].[ExecutionRunStatusReport]'
END
GO

--
--
--
CREATE PROCEDURE [ETLFramework].[ExecutionRunStatusReport] 
AS
BEGIN
SELECT 
	[a].[ExecutionJobLogID]
	, [c].[ExecutionJobLogStart]
	, [b].[JobMasterID]
	, [b].[JobMasterName]

	, CASE WHEN ([c].[ExecutionJobLogStop] IS NULL) THEN DATEDIFF( SECOND, [c].[ExecutionJobLogStart], GETDATE() )
		ELSE DATEDIFF( SECOND, [c].[ExecutionJobLogStart], [c].[ExecutionJobLogStop] )
	END AS [ExecutionJobDurationSeconds]

	, [c].[ExecutionJobLogStatusID]


	, COUNT( [a].[ExecutionLogID] ) AS [TotalCount]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 0 THEN 1 ELSE 0 END ) AS [TotalNotRun]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 1 THEN 1 ELSE 0 END ) AS [TotalRunning]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 2 THEN 1 ELSE 0 END ) AS [TotalSuccess]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 3 THEN 1 ELSE 0 END ) AS [TotalError]
FROM [ETLFramework].[ExecutionLog] [a] WITH (NOLOCK)
JOIN [ETLFramework].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
JOIN [ETLFramework].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]

GROUP BY
	[a].[ExecutionJobLogID]
	, [c].[ExecutionJobLogStart]
	, [b].[JobMasterID]
	, [b].[JobMasterName]
	, [c].[ExecutionJobLogStop]
	, [c].[ExecutionJobLogStatusID]

ORDER BY
	[a].[ExecutionJobLogID]
	, [c].[ExecutionJobLogStart]
	, [b].[JobMasterID]
	, [b].[JobMasterName]
	, [c].[ExecutionJobLogStatusID]

END
GO
PRINT 'CREATE PROCEDURE [ETLFramework].[ExecutionRunStatusReport]'
