
IF db_id('ETL') is null
BEGIN
  Print ('ETL Database does not exist...exiting script')
  set noexec on;
END
GO

IF NOT EXISTS ( SELECT schema_name FROM information_schema.SCHEMATA where schema_name = 'RTLS_NDRDW')
BEGIN
  Print('RTLS_NDRDW SCHEMA DOES NOT EXIST...exiting script')
  set noexec on;
END
GO 

USE [ETL]
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptAggregeteDiskSpaceHistory]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptAggregeteDiskSpaceHistory]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptAggregeteDiskSpaceHistory]
GO
-- =============================================
-- Author:		Michael D. Hanson
--				HPES
--				michael.d.hanson@hp.com
-- Create date: 01/15/2015
-- Description:	creates an aggregate on Historical
--				disk space usage
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptAggregeteDiskSpaceHistory] --'04/17/2015', '04/18/2015'
	@StartDate datetime = NULL,
	@EndDate datetime = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;
	IF @StartDate is NULL
	BEGIN
		SET @StartDate = GetDate() - 30
	END
	IF @EndDate is NULL
	BEGIN
		SET @EndDate = GetDate()
	END
	select	cast(pulldate as smalldatetime) as DatePulled
			, count(dbname) as NumberDatabases
			, Sum(space_used_mb)/1024 as DiskSpaceUsedGB
			, Sum(PercDiskUsed) as PercentDiskSpaceUsed
			, Avg(TotalDiskSpaceMB)/1024 as TotalDiskSpaceGB
			, Min(AvailDiskSpaceMB)/1024 as AvailDiskSpaceGB
	from [RTLS_NDRDW].[rptDatabaseDiskSpaceHistory]
	where pulldate between @StartDate and @EndDate
	group by PullDate
END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptCaptureCPU_UtilData]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptCaptureCPU_UtilData]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptCaptureCPU_UtilData]
GO

CREATE procedure [RTLS_NDRDW].[rptCaptureCPU_UtilData] as 
BEGIN

DECLARE @ts_now bigint = (SELECT cpu_ticks/(cpu_ticks/ms_ticks)FROM sys.dm_os_sys_info); 
with CPUData as
(

SELECT TOP(256) SQLProcessUtilization AS [SQLCPUUtilization]
				,SystemIdle AS [SystemIdleProcess]
				,100 - SystemIdle - SQLProcessUtilization AS [SystemCPUUtilization]
				,100 - SystemIdle AS [TotalCPUUtilization]
				,DATEADD(ms, -1 * (@ts_now - [timestamp]), GETDATE()) AS [EventTime] 
FROM ( 
	  SELECT record.value('(./Record/@id)[1]', 'int') AS record_id, 
			record.value('(./Record/SchedulerMonitorEvent/SystemHealth/SystemIdle)[1]', 'int') 
			AS [SystemIdle], 
			record.value('(./Record/SchedulerMonitorEvent/SystemHealth/ProcessUtilization)[1]', 'int') 
			AS [SQLProcessUtilization], [timestamp] 
	  FROM ( 
			SELECT [timestamp], CONVERT(xml, record) AS [record] 
			FROM sys.dm_os_ring_buffers WITH (NOLOCK)
			WHERE ring_buffer_type = N'RING_BUFFER_SCHEDULER_MONITOR' 
			AND record LIKE N'%<SystemHealth>%') AS x 
	  ) AS y )
INSERT INTO [RTLS_NDRDW].[rptCPUUtilizationHistory]
select CAST(CONVERT(CHAR(17), EventTime, 113) AS datetime) as EventTime
	,SystemCPUUtilization
	,SQLCPUUtilization
	,SystemIdleProcess
	,TotalCPUUtilization
--INTO ssisdb.dbo.CPUUtilization
from CPUData c
where CAST(CONVERT(CHAR(17), EventTime, 113) AS datetime) > 
   (select isnull(max(EventTime),getdate()-1) from [RTLS_NDRDW].[rptCPUUtilizationHistory])
order by EventTime DESC
END


GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptCaptureDBDiskSpaceHistory]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptCaptureDBDiskSpaceHistory]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptCaptureDBDiskSpaceHistory]
GO
-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/08/2015
-- Description:	This procedure will correlate
--				and store database disk space 
--				usage
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptCaptureDBDiskSpaceHistory] 
AS
BEGIN

	SET NOCOUNT ON;

------------------------------Data file size----------------------------
--if exists (select * from tempdb.sys.all_objects where name like '%#dbsize%')
--drop table #dbsize
create table #dbsize
	(dbname	sysname,
	DID int,
	fileid int,
			dbstatus varchar(50),
			Recovery_Model varchar(40) default ('NA'), 
			file_Size_MB decimal(30,2)default (0),
			Space_Used_MB decimal(30,2)default (0),
			Free_Space_MB decimal(30,2) default (0))

 
insert into #dbsize(Dbname,did, fileid, dbstatus,Recovery_Model,file_Size_MB,Space_Used_MB,Free_Space_MB)
exec sp_msforeachdb
'use [?];
	select	DB_NAME() AS DbName
			,DB_ID() AS DID
			,file_id as fileid
			,CONVERT(varchar(20),DatabasePropertyEx(''?'',''Status''))
			,CONVERT(varchar(20),DatabasePropertyEx(''?'',''Recovery''))
			,sum(size)/128.0 AS File_Size_MB
			,sum(CAST(FILEPROPERTY(name, ''SpaceUsed'') AS INT))/128.0 as Space_Used_MB
			,SUM( size)/128.0 - sum(CAST(FILEPROPERTY(name,''SpaceUsed'') AS INT))/128.0 AS Free_Space_MB 
from sys.database_files  --CROSS APPLY sys.dm_os_volume_stats(DB_ID(sys.database_files.name), sys.database_files.file_id)
where type=0 group by type, file_id'
 
-------------------log size--------------------------------------
--if exists (select * from tempdb.sys.all_objects where name like '#logsize%')
--	drop table #logsize
create table #logsize
	(Dbname sysname,
	did int,
	fileid int, 
	Log_File_Size_MB decimal(38,2)default (0),
	log_Space_Used_MB decimal(30,2)default (0),
	log_Free_Space_MB decimal(30,2)default (0))

insert into #logsize(Dbname, did, fileid, Log_File_Size_MB,log_Space_Used_MB,log_Free_Space_MB)
exec sp_msforeachdb
'use [?];
	select	DB_NAME() AS DbName
			,DB_ID() AS did
			,file_id as fileid
			,sum(size)/128.0 AS Log_File_Size_MB
			,sum(CAST(FILEPROPERTY(name, ''SpaceUsed'') AS INT))/128.0 as log_Space_Used_MB
			,SUM( size)/128.0 - sum(CAST(FILEPROPERTY(name,''SpaceUsed'') AS INT))/128.0 AS log_Free_Space_MB 
from sys.database_files  
where type=1 group by type, file_id'
 
 

--------------------------------database free size
--if exists (select * from tempdb.sys.all_objects where name like '%#dbfreesize%')
--	drop table #dbfreesize
create table #dbfreesize
	(name sysname,
	did int,
	fileid int,
	database_size varchar(50),
	Freespace varchar(50)default (0.00))
 
insert into #dbfreesize(name, database_size,Freespace)
exec sp_msforeachdb
'use [?];
	SELECT	database_name = db_name()
			,database_size = ltrim(str((convert(DECIMAL(15, 2), dbsize) + convert(DECIMAL(15, 2), logsize)) * 8192 / 1048576, 15, 2) + ''MB'')
			,''unallocated space'' = ltrim(str((
                CASE 
                    WHEN dbsize >= reservedpages
                        THEN (convert(DECIMAL(15, 2), dbsize) - convert(DECIMAL(15, 2), reservedpages)) * 8192 / 1048576
                    ELSE 0
                    END
                ), 15, 2) + '' MB'')
	FROM (SELECT dbsize = sum(convert(BIGINT, CASE 
                    WHEN type = 0
                        THEN size
                    ELSE 0
                    END))
			,logsize = sum(convert(BIGINT, CASE 
                    WHEN type <> 0
                        THEN size
                    ELSE 0
                    END))
			FROM sys.database_files
	) AS files
,(
    SELECT reservedpages = sum(a.total_pages)
        ,usedpages = sum(a.used_pages)
        ,pages = sum(CASE 
                WHEN it.internal_type IN (
                        202
                        ,204
                        ,211
                        ,212
                        ,213
                        ,214
                        ,215
                        ,216
                        )
                    THEN 0
                WHEN a.type <> 1
                    THEN a.used_pages
                WHEN p.index_id < 2
                    THEN a.data_pages
                ELSE 0
                END)
    FROM sys.partitions p
    INNER JOIN sys.allocation_units a
        ON p.partition_id = a.container_id
    LEFT JOIN sys.internal_tables it
        ON p.object_id = it.object_id
) AS partitions'
-----------------------------------
 
 
 
--if exists (select * from tempdb.sys.all_objects where name like '%#alldbstate%')
--	drop table #alldbstate 
create table #alldbstate 
	(dbname sysname,
	 did int,
	 fileid int,
	DBstatus varchar(55),
	R_model Varchar(30))
  
--select * from sys.master_files
 
insert into #alldbstate (dbname,DBstatus,R_model)
select name,CONVERT(varchar(20),DATABASEPROPERTYEX(name,'status')),recovery_model_desc from sys.databases
--select * from #dbsize
 
insert into #dbsize(Dbname,dbstatus,Recovery_Model)
select dbname,dbstatus,R_model from #alldbstate where DBstatus <> 'online'
 
insert into #logsize(Dbname)
select dbname from #alldbstate where DBstatus <> 'online'
 
insert into #dbfreesize(name)
select dbname from #alldbstate where DBstatus <> 'online'

insert into [RTLS_NDRDW].[rptDatabaseDiskSpaceHistory] 
select	d.Dbname
		,d.did
		,d.fileid
		,d.dbstatus
		,d.Recovery_Model,
		(file_size_mb + log_file_size_mb) as DBsize,
		d.file_Size_MB
		,d.Space_Used_MB
		,d.Free_Space_MB,
		l.Log_File_Size_MB
		,log_Space_Used_MB
		,l.log_Free_Space_MB
		,fs.Freespace as DB_Freespace
		,(d.space_used_mb) / (total_bytes/1024/1024) * 100 as PercDiskUsed
		,(total_bytes/1024/1024) as TotalDiskSpaceMB
		,(available_bytes/1024/1024) as AvailDiskSpaceMB
		,getdate() as PullDate
--into [ETLDashboard].[dbo].[rptDatabaseDiskSpace]
from #dbsize d join #logsize l on d.Dbname=l.Dbname 
	join #dbfreesize fs on d.Dbname=fs.name
	CROSS APPLY sys.dm_os_volume_stats(d.did, d.fileid)
--group by d.dbname, d.did, d.fileid, d.dbstatus, d.recovery_model, d.file_size_mb, d.free_space_mb, l.log_file_size_mb, log_space_used_mb
--	,l.log_free_space_mb, fs.freespace
order by Dbname

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptCaptureMemoryStats]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptCaptureMemoryStats]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptCaptureMemoryStats]
GO


CREATE procedure [RTLS_NDRDW].[rptCaptureMemoryStats]
as
BEGIN
-- Good basic information about memory amounts and state
INSERT into [RTLS_NDRDW].[rptMemoryUtilizationHistory]
SELECT	CAST(CONVERT(CHAR(17), GetDate(), 113) AS datetime) as EventTime
		,total_physical_memory_kb/1024 as TotalPhysMemMB
		,available_physical_memory_kb/1024 as AvailPhysMemMB
		,total_page_file_kb/1024 as TotalPageFileMB
		,available_page_file_kb/1024 as AvailPageFileMB
		,system_memory_state_desc
FROM sys.dm_os_sys_memory WITH (NOLOCK) OPTION (RECOMPILE)

END


GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptCaptureSQLPerfMonData]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptCaptureSQLPerfMonData]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptCaptureSQLPerfMonData]
GO

-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/11/2015
-- Description:	Captures SQL performance 
--				monitoring stats for reporting
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptCaptureSQLPerfMonData] 
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

   -- Insert statements for procedure here
	DECLARE @SQLProcessUtilization int;
	DECLARE @PageReadsPerSecond bigint
	DECLARE @PageWritesPerSecond bigint
	DECLARE @PageSplitsPerSecond bigint
	DECLARE @CheckpointPagesPerSecond bigint
	DECLARE @LazyWritesPerSecond bigint
	DECLARE @BatchRequestsPerSecond bigint
	DECLARE @CompilationsPerSecond bigint
	DECLARE @ReCompilationsPerSecond bigint
	DECLARE @PageLookupsPerSecond bigint
	DECLARE @TransactionsPerSecond bigint
	DECLARE @stat_date datetime

	--get page splits before other counters to filter 
	--for true page splits
	DECLARE @old_cntr_value INT;
	DECLARE @first_sample_date DATETIME;
	SELECT	@old_cntr_value = cntr_value,
			@first_sample_date = getdate()
	FROM sys.dm_os_performance_counters
	WHERE counter_name = 'Page Splits/sec';

	-- Time frame to wait before collecting second sample
	WAITFOR DELAY '00:00:05';

	-- Collect second sample and calculate per-second counter
	SELECT @PageSplitsPerSecond = (cntr_value - @old_cntr_value) / 	DATEDIFF(ss,@first_sample_date, GETDATE()) 
	FROM sys.dm_os_performance_counters
	WHERE counter_name = 'Page Splits/sec'
			AND object_name = CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
									THEN 'SQLServer:Access Methods'
									ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Access Methods' 
							  END

	-- Table for First Sample
	DECLARE @RatioStatsX 
	TAbLE([object_name] varchar(128)
			,[counter_name] varchar(128)
			,[instance_name] varchar(128)
			,[cntr_value] bigint
			,[cntr_type] int
	)

	-- Table for Second Sample
	DECLARE @RatioStatsY 
	TAbLE([object_name] varchar(128)
			,[counter_name] varchar(128)
			,[instance_name] varchar(128)
			,[cntr_value] bigint
			,[cntr_type] int
	)

	INSERT INTO @RatioStatsX (
		[object_name]
		,[counter_name]
		,[instance_name]
		,[cntr_value]
		,[cntr_type] )
	SELECT	[object_name]
			,[counter_name]
			,[instance_name]
			,[cntr_value]
			,[cntr_type] FROM sys.dm_os_performance_counters

	SET @stat_date = getdate()
	SELECT TOP 1 @PageReadsPerSecond=cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Page reads/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Buffer Manager'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END

	SELECT TOP 1 @PageWritesPerSecond= cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Page writes/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Buffer Manager'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END

	SELECT TOP 1 @CheckpointPagesPerSecond = cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Checkpoint pages/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Buffer Manager'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END

	SELECT TOP 1 @LazyWritesPerSecond = cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Lazy writes/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Buffer Manager'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END

	SELECT TOP 1 @BatchRequestsPerSecond = cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Batch Requests/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:SQL Statistics'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':SQL Statistics' 
							END

	SELECT TOP 1 @CompilationsPerSecond = cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'SQL Compilations/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:SQL Statistics'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':SQL Statistics' 
							END

	SELECT TOP 1 @ReCompilationsPerSecond = cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'SQL Re-Compilations/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:SQL Statistics'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':SQL Statistics' 
							END

	SELECT TOP 1 @PageLookupsPerSecond=cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Page lookups/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Buffer Manager'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END

	SELECT TOP 1 @TransactionsPerSecond=cntr_value
	FROM @RatioStatsX
	WHERE counter_name = 'Transactions/sec' AND instance_name = '_Total'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Databases'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Databases' 
							END

	-- Wait for 5 seconds before taking second sample
	WAITFOR DELAY '00:00:05'

	-- Table for second sample
	INSERT INTO @RatioStatsY (
			[object_name]
			,[counter_name]
			,[instance_name]
			,[cntr_value]
			,[cntr_type] )
	SELECT	[object_name]
			,[counter_name]
			,[instance_name]
			,[cntr_value]
			,[cntr_type] FROM sys.dm_os_performance_counters

	INSERT INTO [RTLS_NDRDW].[rptSQLPerfMonData]
	SELECT (a.cntr_value * 1.0 / b.cntr_value) * 100.0 [BufferCacheHitRatio]
			,c.[PageReadPerSec] [PageReadsPerSec]
			,d.[PageWritesPerSecond] [PageWritesPerSecond]
			,@PageSplitsPerSecond [PageSplitsPerSecond]
			,e.cntr_value [UserConnections]
			,f.cntr_value [PageLifeExpectency]
			,g.[CheckpointPagesPerSecond] [CheckpointPagesPerSecond]
			,h.[LazyWritesPerSecond] [LazyWritesPerSecond]
			,i.cntr_value [FreeSpaceInTempdbKB]
			,j.[BatchRequestsPerSecond] [BatchRequestsPerSecond]
			,k.[SQLCompilationsPerSecond] [SQLCompilationsPerSecond]
			,l.[SQLReCompilationsPerSecond] [SQLReCompilationsPerSecond]
			,m.cntr_value [Target Server Memory (KB)]
			,n.cntr_value [Total Server Memory (KB)]
			,GETDATE() AS [MeasurementTime]
			,o.[AvgTaskCount]
			,o.[AvgRunnableTaskCount]
			,o.[AvgPendingDiskIOCount]
			,p.PercentSignalWait AS [PercentSignalWait]
			,q.PageLookupsPerSecond As [PageLookupsPerSecond]
			,r.TransactionsPerSecond AS [TransactionsPerSecond]
			,s.cntr_value [MemoryGrantsPending]
			,t.cntr_value [LockWaitsPerSecondTotal]
			,u.cntr_value [ProcessesBlocked]
	FROM (SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Buffer cache hit ratio'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END ) a
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Buffer cache hit ratio base'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END ) b on a.x = b.x
	join
	(SELECT (cntr_value - @PageReadsPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [PageReadPerSec], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Page reads/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END)c on a.x = c.x
	join
	(SELECT (cntr_value - @PageWritesPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [PageWritesPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Page writes/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END) d on a.x = d.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'User Connections'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:General Statistics' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':General Statistics' 
							END ) e on a.x = e.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Page life expectancy '
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END ) f on a.x = f.x
	join
	(SELECT (cntr_value - @CheckpointPagesPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [CheckpointPagesPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Checkpoint pages/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END) g on a.x = g.x
	join
	(SELECT (cntr_value - @LazyWritesPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [LazyWritesPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Lazy writes/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
							END) h on a.x = h.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Free Space in tempdb (KB)'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Transactions' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Transactions' 
							end) i on a.x = i.x
	join
	(SELECT (cntr_value - @BatchRequestsPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [BatchRequestsPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Batch Requests/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:SQL Statistics' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':SQL Statistics' 
							END) j on a.x = j.x
	join
	(SELECT (cntr_value - @CompilationsPerSecond) / (CASE WHEN datediff(ss,@stat_date,getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [SQLCompilationsPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'SQL Compilations/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:SQL Statistics' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':SQL Statistics' 
								END) k on a.x = k.x
	join
	(SELECT (cntr_value - @ReCompilationsPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [SQLReCompilationsPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'SQL Re-Compilations/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:SQL Statistics' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':SQL Statistics' 
							END) l on a.x = l.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Target Server Memory (KB)'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Memory Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Memory Manager' 
							END ) m on a.x = m.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Total Server Memory (KB)'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Memory Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Memory Manager' 
							END) n	on a.x = n.x
	JOIN
	(SELECT 1 AS x
			, AVG(current_tasks_count)AS [AvgTaskCount]
			, AVG(runnable_tasks_count)AS [AvgRunnableTaskCount]
			, AVG(pending_disk_io_count) AS [AvgPendingDiskIOCount]
	FROM sys.dm_os_schedulers
	WHERE scheduler_id < 255) o on a.x = o.x
	JOIN
	( SELECT 1 AS x, SUM(signal_wait_time_ms) / sum (wait_time_ms) AS PercentSignalWait
	FROM sys.dm_os_wait_stats) p ON a.x = p.x
	join
	(SELECT (cntr_value - @PageLookupsPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [PageLookupsPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Page Lookups/sec'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Buffer Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Buffer Manager' 
								END) q	on a.x = q.x
	join
	(SELECT (cntr_value - @TransactionsPerSecond) / (CASE WHEN datediff(ss,@stat_date, getdate()) = 0 THEN 1 
		ELSE datediff(ss,@stat_date, getdate()) end) as [TransactionsPerSecond], 1 x
	FROM @RatioStatsY
	WHERE counter_name = 'Transactions/sec' AND instance_name = '_Total'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
								THEN 'SQLServer:Databases'
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Databases' 
							END ) r	on a.x = r.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Memory Grants Pending'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Memory Manager' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Memory Manager' 
							END ) s on a.x = s.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Lock Waits/sec' and instance_name = '_Total'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:Locks' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':Locks' 
							END ) t on a.x = t.x
	join
	(SELECT *, 1 x FROM @RatioStatsY
	WHERE counter_name = 'Processes Blocked'
		AND object_name =	CASE WHEN @@SERVICENAME = 'MSSQLSERVER' 
								THEN 'SQLServer:General Statistics' 
								ELSE 'MSSQL$' + rtrim(@@SERVICENAME) + ':General Statistics' 
								END ) u on a.x = u.x



END


GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptDashBoardCounts]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptDashBoardCounts]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptDashBoardCounts]
GO


-- =============================================
-- Author:		Michael D. Hanson
--				HP Enterprise Services
-- Create date: 04-15-2015
-- Description:	This procedure provides job 
--				counts for the ETL Framework
--				Dashboard
-- Parameters:	@StartDate - beginning date range
--				@EndDate - ending date range
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptDashBoardCounts] --'04/01/2015', '04/16/2015'
@StartDate as DateTime,
@EndDate as DateTime
as 
BEGIN


BEGIN
	WITH e AS 
	(SELECT [a].[ExecutionLogStatusID] as [status], 
			[b].[JobMasterName] as [folder_name], 
			COUNT([a].[ExecutionLogStatusID]) AS StatCount
	FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
		JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
		JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
	WHERE [c].[ExecutionJobLogStart] BETWEEN @StartDate AND @EndDate 
	GROUP BY [a].[ExecutionLogStatusID], [b].[JobMasterName]
	),
	p AS
	(SELECT [b].[JobMasterName] AS pkgPath
	FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
		JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
		JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
	WHERE (CONVERT(DATETIME, [c].[ExecutionJobLogStart]) BETWEEN @StartDate AND @EndDate)
	)
	    
	SELECT 'dd' AS DrillDown, 
			[folder_name], 
			ISNULL(MAX(CASE [status] WHEN 3 THEN StatCount ELSE 0 END), '0') AS 'Failed', 
			MAX(CASE [status] WHEN 1 THEN StatCount ELSE 0 END) AS 'Running', 
			MAX(CASE [status] WHEN 2 THEN StatCount ELSE 0 END) AS 'Success', 
			MAX(CASE [status] WHEN 0 THEN StatCount ELSE 0 END) AS 'Other', 
			SUM(StatCount) AS StatCount,
			(select COUNT(DISTINCT([pkgPath])) from p) as nDistinctPackages,
			(SELECT COUNT(1) AS Expr1 FROM [RTLS_NDRDW].[JobMaster]) AS nAllPackages
	FROM e AS e_1
	GROUP BY folder_name
	ORDER BY folder_name
END

END
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptDashBoardExecution]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptDashBoardExecution]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptDashBoardExecution]
GO


-- =============================================
-- Author:		Michael D. Hanson
--				HP Enterprise Services
-- Create date: 04-15-2015
-- Description:	This procedure provides job 
--				execution data for the ETL
--				Dashboard
-- Parameters:	@StartDate - beginning date range
--				@EndDate - ending date range
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptDashBoardExecution] --'04/01/2015','04/16/2015'
@StartDate as DateTime,
@EndDate as DateTime,
@StatusCode as int
as
BEGIN

DECLARE @xStartDate DateTime, @xEndDate as DateTime, @xStatusCode as int
SET @xStartDate = @StartDate
SET @xEndDate = @EndDate
SET @xStatusCode = @StatusCode

	SELECT	 [EL].[ExecutionLogStatusID]
			,[EL].[execution_id]
			,[PM].[PackageFolder]
			,[PM].[PackageProject]
			,[PM].[PackageName]
			,[EL].[ExecutionLogStart]
			,[EL].[ExecutionLogStop]
			,DATEDIFF( SECOND, [EJL].[ExecutionJobLogStart], [EJL].[ExecutionJobLogStop] ) AS Duration
			,[JM].[JobMasterName]
			,[FM].[FlightName]
			,[VM].[VISNName]
			,[VM].[VISNLongName]
			,CASE WHEN LEN(op.[message]) <= 4096 THEN op.[message] ELSE LEFT(op.[message], 1024) + '...' END AS [message]
	FROM [RTLS_NDRDW].[ExecutionLog] [EL] with (nolock)
		   INNER JOIN [RTLS_NDRDW].[VISNMaster] [VM] with (nolock) on [EL].[VISNMasterID] = [VM].[VISNMasterID]
		   INNER JOIN [RTLS_NDRDW].[JobMaster] [JM] with (nolock) on [EL].[JobMasterID] = [JM].[JobMasterID]
		   INNER JOIN [RTLS_NDRDW].[FlightMaster] [FM] with (nolock) on [EL].[FlightMasterID] = [FM].[FlightMasterID]
		   INNER JOIN [RTLS_NDRDW].[PackageMaster] [PM] with (nolock) on [EL].[PackageMasterID] = [PM].[PackageMasterID]
		   INNER JOIN [RTLS_NDRDW].[ExecutionJobLog] [EJL] with (nolock) on [EL].[ExecutionJobLogID] = [EJL].[ExecutionJobLogID]
		   INNER JOIN [ssisdb].[internal].[event_messages] msg ON msg.[operation_id] = [EL].[execution_id]
		   INNER JOIN [SSISDB].[internal].[operation_messages] op ON op.[operation_message_id] = msg.[event_message_id]
		   --INNER JOIN [RTLS_NDRDW].[ExecutionCode] [EC] on [EL].[ExecutionLogStatusID] = [EC].[CodeID]
	WHERE	[EL].[ExecutionLogStatusID] = @xStatusCode
			AND [EJL].[ExecutionJobLogStart] BETWEEN @xStartDate and @xEndDate
			AND op.[message_type] = 120

END


GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptDBDiskSpace]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptDBDiskSpace]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptDBDiskSpace]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/08/2015
-- Description:	This procedure will show
--				database disk space usage
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptDBDiskSpace] 
AS
BEGIN

	SET NOCOUNT ON;

------------------------------Data file size----------------------------
--if exists (select * from tempdb.sys.all_objects where name like '%#dbsize%')
--drop table #dbsize
create table #dbsize
	(dbname	sysname,
	DID int,
	fileid int,
			dbstatus varchar(50),
			Recovery_Model varchar(40) default ('NA'), 
			file_Size_MB decimal(30,2)default (0),
			Space_Used_MB decimal(30,2)default (0),
			Free_Space_MB decimal(30,2) default (0))

 
insert into #dbsize(Dbname,did, fileid, dbstatus,Recovery_Model,file_Size_MB,Space_Used_MB,Free_Space_MB)
exec sp_msforeachdb
'use [?];
	select	DB_NAME() AS DbName
			,DB_ID() AS DID
			,file_id as fileid
			,CONVERT(varchar(20),DatabasePropertyEx(''?'',''Status''))
			,CONVERT(varchar(20),DatabasePropertyEx(''?'',''Recovery''))
			,sum(size)/128.0 AS File_Size_MB
			,sum(CAST(FILEPROPERTY(name, ''SpaceUsed'') AS INT))/128.0 as Space_Used_MB
			,SUM( size)/128.0 - sum(CAST(FILEPROPERTY(name,''SpaceUsed'') AS INT))/128.0 AS Free_Space_MB 
from sys.database_files  --CROSS APPLY sys.dm_os_volume_stats(DB_ID(sys.database_files.name), sys.database_files.file_id)
where type=0 group by type, file_id'
 
-------------------log size--------------------------------------
--if exists (select * from tempdb.sys.all_objects where name like '#logsize%')
--	drop table #logsize
create table #logsize
	(Dbname sysname,
	did int,
	fileid int, 
	Log_File_Size_MB decimal(38,2)default (0),
	log_Space_Used_MB decimal(30,2)default (0),
	log_Free_Space_MB decimal(30,2)default (0))

insert into #logsize(Dbname, did, fileid, Log_File_Size_MB,log_Space_Used_MB,log_Free_Space_MB)
exec sp_msforeachdb
'use [?];
	select	DB_NAME() AS DbName
			,DB_ID() AS did
			,file_id as fileid
			,sum(size)/128.0 AS Log_File_Size_MB
			,sum(CAST(FILEPROPERTY(name, ''SpaceUsed'') AS INT))/128.0 as log_Space_Used_MB
			,SUM( size)/128.0 - sum(CAST(FILEPROPERTY(name,''SpaceUsed'') AS INT))/128.0 AS log_Free_Space_MB 
from sys.database_files  
where type=1 group by type, file_id'
 
 

--------------------------------database free size
--if exists (select * from tempdb.sys.all_objects where name like '%#dbfreesize%')
--	drop table #dbfreesize
create table #dbfreesize
	(name sysname,
	did int,
	fileid int,
	database_size varchar(50),
	Freespace varchar(50)default (0.00))
 
insert into #dbfreesize(name, database_size,Freespace)
exec sp_msforeachdb
'use [?];
	SELECT	database_name = db_name()
			,database_size = ltrim(str((convert(DECIMAL(15, 2), dbsize) + convert(DECIMAL(15, 2), logsize)) * 8192 / 1048576, 15, 2) + ''MB'')
			,''unallocated space'' = ltrim(str((
                CASE 
                    WHEN dbsize >= reservedpages
                        THEN (convert(DECIMAL(15, 2), dbsize) - convert(DECIMAL(15, 2), reservedpages)) * 8192 / 1048576
                    ELSE 0
                    END
                ), 15, 2) + '' MB'')
	FROM (SELECT dbsize = sum(convert(BIGINT, CASE 
                    WHEN type = 0
                        THEN size
                    ELSE 0
                    END))
			,logsize = sum(convert(BIGINT, CASE 
                    WHEN type <> 0
                        THEN size
                    ELSE 0
                    END))
			FROM sys.database_files
	) AS files
,(
    SELECT reservedpages = sum(a.total_pages)
        ,usedpages = sum(a.used_pages)
        ,pages = sum(CASE 
                WHEN it.internal_type IN (
                        202
                        ,204
                        ,211
                        ,212
                        ,213
                        ,214
                        ,215
                        ,216
                        )
                    THEN 0
                WHEN a.type <> 1
                    THEN a.used_pages
                WHEN p.index_id < 2
                    THEN a.data_pages
                ELSE 0
                END)
    FROM sys.partitions p
    INNER JOIN sys.allocation_units a
        ON p.partition_id = a.container_id
    LEFT JOIN sys.internal_tables it
        ON p.object_id = it.object_id
) AS partitions'
-----------------------------------
 
 
 
--if exists (select * from tempdb.sys.all_objects where name like '%#alldbstate%')
--	drop table #alldbstate 
create table #alldbstate 
	(dbname sysname,
	 did int,
	 fileid int,
	DBstatus varchar(55),
	R_model Varchar(30))
  
--select * from sys.master_files
 
insert into #alldbstate (dbname,DBstatus,R_model)
select name,CONVERT(varchar(20),DATABASEPROPERTYEX(name,'status')),recovery_model_desc from sys.databases
--select * from #dbsize
 
insert into #dbsize(Dbname,dbstatus,Recovery_Model)
select dbname,dbstatus,R_model from #alldbstate where DBstatus <> 'online'
 
insert into #logsize(Dbname)
select dbname from #alldbstate where DBstatus <> 'online'
 
insert into #dbfreesize(name)
select dbname from #alldbstate where DBstatus <> 'online'

--insert into [ETLDashboard].[dbo].[rptDatabaseDiskSpace] 
select	d.Dbname
		,d.did
		,d.fileid
		,d.dbstatus
		,d.Recovery_Model,
		(file_size_mb + log_file_size_mb) as DBsize,
		d.file_Size_MB
		,d.Space_Used_MB
		,d.Free_Space_MB,
		l.Log_File_Size_MB
		,log_Space_Used_MB
		,l.log_Free_Space_MB
		,fs.Freespace as DB_Freespace
		,(d.space_used_mb) / (total_bytes/1024/1024) * 100 as PercDiskUsed
		,(total_bytes/1024/1024) as TotalDiskSpaceMB
		,(available_bytes/1024/1024) as AvailDiskSpaceMB
		,getdate() as PullDate
--into [ETLDashboard].[dbo].[rptDatabaseDiskSpace]
from #dbsize d join #logsize l on d.Dbname=l.Dbname 
	join #dbfreesize fs on d.Dbname=fs.name
	CROSS APPLY sys.dm_os_volume_stats(d.did, d.fileid)
--group by d.dbname, d.did, d.fileid, d.dbstatus, d.recovery_model, d.file_size_mb, d.free_space_mb, l.log_file_size_mb, log_space_used_mb
--	,l.log_free_space_mb, fs.freespace
order by Dbname

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptEDAReportView]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptEDAReportView]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptEDAReportView]


GO
-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/22/2015
-- Update date: 03/03/2015
--				Updated to utilize ETL Framework
--				Metadata Discovery Data
--
--				03/13/2015
--				Updated to add mulitple flags
--				to show different views
--				added columns to help 
--				differentiate old and new 
--				values
-- Description:	Pull MetaData values for the
--				Data Dictionary report 
--				based on database namve passed
--				in
-- Parameters:	
--				@Database: name of dB to pull
--					values from
--				@Flag: type of report to run
--					1: Current Version 
--					2: Base Version (original values)
--					3: Base + changes (changes to the base shown)
--					4: Changes only (only show changes)
-- =============================================

CREATE  PROCEDURE [RTLS_NDRDW].[rptEDAReportView] --'insites', 3
	-- Add the parameters for the stored procedure here
	@xDatabase varchar(128) = 'insites',
	@xFlag int = 1
AS


DECLARE @Database varchar(128), @Flag int;

SET @Database = @xDatabase
SET @Flag = @xFlag;

BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	DECLARE @BASE date
	SET @BASE = (SELECT CONVERT(DATE, MIN(CreateDate)) FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges]);
	
	IF @Flag = 1 --CURENT VERSION OF REPORT
		BEGIN
			SELECT	sm.SystemName as DatabaseName
				,sm.Description as DatabaseDescription
				,md.TableName
				,md.TableOwner as TableSchema
				,md.ObjectDescription as TableDescription
				,CASE
					WHEN ky.ColumnName is not NULL then ky.ColumnName
					ELSE md.ColumnName
				 END as ColumnName
				,md.CurrentMetadataExtractID as ColumnOrder
				, CASE
					WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
					ELSE ISNULL(md.ObjectDescription, '')
				  END AS ColumnDescription
				, '' as TModID
				, '' as TModTime
				, '' as CModID
				, '' as CModTime
				, '' as DMLID
				, md.DataType
				, md.ColumnLength as [Length]
				, md.DefaultValue
				, ky.ConstraintType as [Key]
				, md.AllowsNulls as NULLSAllowed
				, ky.ConstraintName as KeyName
				, ky.ConstraintType as KeyType
			FROM	[RTLS_NDRDW].[CurrentMetadataExtract] md WITH (NOLOCK)
					JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
					LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
						and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
			WHERE sm.SystemName = @Database 
				  and md.ObjectType is NULL
		    OPTION (OPTIMIZE FOR UNKNOWN);
		END 
	ELSE IF @Flag = 2 --BASE VERSION
		BEGIN
			SELECT	sm.SystemName as DatabaseName
					,sm.Description as DatabaseDescription
					,md.TableName
					,md.TableOwner as TableSchema
					,md.ObjectDescription as TableDescription
					,CASE
						WHEN ky.ColumnName is not NULL then ky.ColumnName
						ELSE md.ColumnName
						END as ColumnName
					,md.CurrentMetadataExtractID as ColumnOrder
					, CASE
						WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
						ELSE ISNULL(md.ObjectDescription, '')
						END AS ColumnDescription
					, '' as TModID
					, '' as TModTime
					, '' as CModID
					, '' as CModTime
					, '' as DMLID
					, md.DataType
					, md.ColumnLength as [Length]
					, md.DefaultValue
					, ky.ConstraintType as [Key]
					, md.AllowsNulls as NULLSAllowed
					, ky.ConstraintName as KeyName
					, ky.ConstraintType as KeyType
			FROM	[RTLS_NDRDW].[ETLMetadataIncrementalChanges] md WITH (NOLOCK)
					JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
					LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
						and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
			WHERE sm.SystemName = @Database 
					and md.IsPulledByMetaTool = 1
					and (md.ObjectType is NULL)
					and CONVERT(DATE, md.CreateDate) = (@BASE)
			OPTION (OPTIMIZE FOR UNKNOWN);
		END
	ELSE IF @Flag = 3 --BASE VERSION WITH CHANGES
	BEGIN
			WITH ColumnModification as (
				SELECT	TableName,
						ColumnName,
						CONVERT(DATE, CreateDate) as ModTime,
						SUM([DMLOperationID]) as ModSUM,
						CASE 
							WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
							WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
							WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
							ELSE ''
						END as ModID
				FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
				WHERE ([IsPulledByMetaTool] = 0) 
					  --and CONVERT(Date, CreateDate) in (@Date, @BASE)
					  and ObjectType is NULL
				GROUP BY TableName, ColumnName ,CONVERT(DATE, CreateDate)
			), TableModification as (
					SELECT	TableName,
						CONVERT(DATE, CreateDate) as ModTime,
						SUM([DMLOperationID]) as ModSUM,
						CASE 
							WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
							WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
							WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
							ELSE ''
						END as ModID
				FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
				WHERE ([IsPulledByMetaTool] = 0) 
					  --and CONVERT(Date, CreateDate) = @Date
					  and ObjectType = 'U'
				GROUP BY TableName, CONVERT(Date, CreateDate)
			)
			SELECT	sm.SystemName as DatabaseName
					,sm.Description as DatabaseDescription
					,md.TableName
					,md.TableOwner as TableSchema
					,md.ObjectDescription as TableDescription
					,CASE
						WHEN ky.ColumnName is not NULL then ky.ColumnName
						ELSE md.ColumnName
					 END as ColumnName
					,md.CurrentMetadataExtractID as ColumnOrder
					, CASE
						WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
						ELSE ISNULL(md.ObjectDescription, '')
					  END AS ColumnDescription
					, t.ModID as TModID
					, t.ModTime as TModTime
					, m.ModID as CModID
					, m.ModTime as CModTime
					, CASE 
						WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 2 and (CONVERT(DATE, md.CreateDate) > @Base))  THEN 'NEW' 
						WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 3 and (CONVERT(DATE, md.CreateDate) > @Base)) THEN 'OLD'
						WHEN (m.ModID is null) THEN ''
						ELSE 'OLD'
					  END as DMLID
					, md.DataType
					, md.ColumnLength as [Length]
					, md.DefaultValue
					, ky.ConstraintType as [Key]
					, md.AllowsNulls as NULLSAllowed
					, ky.ConstraintName as KeyName
					, ky.ConstraintType as KeyType
			FROM	[RTLS_NDRDW].[ETLMetadataIncrementalChanges] md WITH (NOLOCK)
					JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
					LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
						and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
					LEFT JOIN ColumnModification m WITH (NOLOCK) on md.[TableName] = m.[TableName] and md.[ColumnName] = m.[ColumnName]
					LEFT JOIN TableModification t WITH (NOLOCK) on md.[TableName] = t.[TableName]
			WHERE sm.SystemName = @Database 
				  --and CONVERT(DATE, md.CreateDate) = @Date
				  and (md.ObjectType is NULL)
		    OPTION (OPTIMIZE FOR UNKNOWN);
	END
	ELSE
	BEGIN --CHANGES ONLY
		WITH ColumnModification as (
			SELECT	TableName,
					ColumnName,
					CONVERT(DATE, CreateDate) as ModTime,
					SUM([DMLOperationID]) as ModSUM,
					CASE 
						WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
						WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
						WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
						ELSE ''
					END as ModID
			FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
			WHERE ([IsPulledByMetaTool] = 0) 
					--and CONVERT(Date, CreateDate) in (@Date, @BASE)
					and ObjectType is NULL
			GROUP BY TableName, ColumnName ,CONVERT(DATE, CreateDate)
		), TableModification as (
				SELECT	TableName,
					CONVERT(DATE, CreateDate) as ModTime,
					SUM([DMLOperationID]) as ModSUM,
					CASE 
						WHEN SUM([DMLOperationID])%2 = 0 THEN 'INS'
						WHEN SUM([DMLOperationID])%3 = 0 THEN 'DEL'
						WHEN SUM([DMLOperationID])%5 = 0 THEN 'UPD'
						ELSE ''
					END as ModID
			FROM [RTLS_NDRDW].[ETLMetadataIncrementalChanges] WITH (NOLOCK)
			WHERE ([IsPulledByMetaTool] = 0) 
					--and CONVERT(Date, CreateDate) = @Date
					and ObjectType = 'U'
			GROUP BY TableName, CONVERT(Date, CreateDate)
		)
		SELECT	sm.SystemName as DatabaseName
				,sm.Description as DatabaseDescription
				,md.TableName
				,md.TableOwner as TableSchema
				,md.ObjectDescription as TableDescription
				,CASE
					WHEN ky.ColumnName is not NULL then ky.ColumnName
					ELSE md.ColumnName
					END as ColumnName
				,md.CurrentMetadataExtractID as ColumnOrder
				, CASE
					WHEN md.DefaultValue <> '' THEN ISNULL(md.ObjectDescription + '<BR>', '') + '<font color = ''blue'' > Initial Value: ' + md.DefaultValue  + '</font>'
					ELSE ISNULL(md.ObjectDescription, '')
					END AS ColumnDescription
				, t.ModID as TModID
				, t.ModTime as TModTime
				, m.ModID as CModID
				, m.ModTime as CModTime
				, CASE 
					WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 2 and (CONVERT(DATE, md.CreateDate) > @Base))  THEN 'NEW' 
					WHEN (m.ModID = 'UPD'AND md.DMLOperationID = 3 and (CONVERT(DATE, md.CreateDate) > @Base)) THEN 'OLD'
					WHEN (m.ModID is null) THEN ''
					ELSE ''
				  END as DMLID
				, md.DataType
				, md.ColumnLength as [Length]
				, md.DefaultValue
				, ky.ConstraintType as [Key]
				, md.AllowsNulls as NULLSAllowed
				, ky.ConstraintName as KeyName
				, ky.ConstraintType as KeyType
		FROM	[RTLS_NDRDW].[ETLMetadataIncrementalChanges] md WITH (NOLOCK)
				JOIN [RTLS_NDRDW].[SystemMaster] sm WITH (NOLOCK) on sm.[SystemMasterID] = md.[SystemMasterID] 
				LEFT JOIN [RTLS_NDRDW].[KeyColumnUsage] ky WITH (NOLOCK) on sm.[SystemMasterID] = ky.[SystemMasterID]
					and md.[TableName] = ky.[TableName] and md.[ColumnName] = ky.[ColumnName]
				LEFT JOIN ColumnModification m WITH (NOLOCK) on md.[TableName] = m.[TableName] and md.[ColumnName] = m.[ColumnName]
				LEFT JOIN TableModification t WITH (NOLOCK) on md.[TableName] = t.[TableName]
		WHERE sm.SystemName = @Database 
				and md.IsPulledByMetaTool = 0
				and (md.ObjectType is NULL)
		OPTION (OPTIMIZE FOR UNKNOWN);
	END


END





GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptETLKPIData]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptETLKPIData]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptETLKPIData]
GO

-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/08/2015
-- Description:	get KPI data for report
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptETLKPIData] --'09/01/2014', '04/15/2015'
@StartDate as datetime,
@EndDate as datetime
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	select 'Average Run Time (minutes)' as Indicator
			,avg(datediff(mi, start_time, end_time)) as stat
			,2 as KPI
	from  [SSISDB].[internal].[execution_info] 
	Where start_time >= @StartDate and end_time <= @EndDate 
		AND folder_name like 'RLTS%'
	UNION
	select 'Max Run Time (minutes)' as Indicator
			,max(datediff(mi, start_time, end_time)) as stat
			,case 
				when max(datediff(mi, start_time, end_time)) >= 10 then 3
				when max(datediff(mi, start_time, end_time)) > 5 and max(datediff(mi, start_time, end_time)) < 10 then 2
				else 1
			 end as KPI

	from  [SSISDB].[internal].[execution_info] 
	Where start_time >= @StartDate and end_time <= @EndDate 
		AND folder_name like 'RLTS%'
	UNION
	select 'Number Failed Jobs' as Indicator
			,count(status) as stat
			,case
				when count(status) >= 100 then 3
				when count(status) > 20 and count(status) < 100 then 2
				else 1
			end as KPI
	from  [SSISDB].[internal].[execution_info] 
	where status = 4 and
		start_time >= @StartDate and end_time <= @EndDate 
		AND folder_name like 'RLTS%'
	UNION
	select 'Number Successful Jobs' as Indicator
			,count(status) as stat
			, 1 as KPI
	from  [SSISDB].[internal].[execution_info] 
	where status in (7, 8) and
		start_time >= @StartDate and end_time <= @EndDate 
		AND folder_name like 'RLTS%'
	UNION
	select 'Number Jobs still running' as Indicator
			,count(status) as stat
			,case 
				when count(status) > 0 then 3
				else 1
			end as KPI
	from  [SSISDB].[internal].[execution_info] 
	where status = 2
	and start_time >= @StartDate and end_time <= @EndDate 
	AND folder_name like 'RLTS%'

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionAllMsgInfo]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionAllMsgInfo]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionAllMsgInfo]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionAllMsgInfo] ---1, 63198
	-- Add the parameters for the stored procedure here
	@MessageType Int = -1, 
	@ExecutionID Int, 
	@SourceNameContains NVarChar(max) = N'', 
	@MessageContains NVarChar(max) = N'', 
	@SubComponentNameContains NVarChar(max) = N'', 
	@ExecutionPathContains NVarChar(max) = N'', 
	@EnvironmentContains NVarChar(max) = N''

AS
BEGIN

WITH msgEx AS(
    SELECT
        msg.[event_message_id]
        ,msg.[operation_id]
        ,CONVERT(datetime, op.[message_time]) AS message_time
        ,op.[message_type]
        ,op.[message_source_type]
        ,CASE WHEN LEN(op.[message]) <= 4096 THEN op.[message] ELSE LEFT(op.[message], 1024) + '...' END AS [message]
        ,op.[extended_info_id]
        ,msg.[event_name]
        ,CASE WHEN LEN(msg.[message_source_name]) <= 1024 THEN msg.[message_source_name] ELSE LEFT(msg.[message_source_name], 1024) + '...' END AS [message_source_name]
        ,msg.[message_source_id]
        ,CASE WHEN LEN(msg.[subcomponent_name]) <= 1024 THEN msg.[subcomponent_name] ELSE LEFT(msg.[subcomponent_name], 1024) + '...' END AS [subcomponent_name]
        ,CASE WHEN LEN(msg.[package_path]) <= 1024 THEN msg.[package_path] ELSE LEFT(msg.[package_path], 1024) + '...' END AS [package_path]
        ,CASE WHEN LEN(msg.[execution_path]) <= 1024 THEN msg.[execution_path] ELSE LEFT(msg.[execution_path], 1024) + '...' END AS [execution_path]
        ,msg.[message_code]
        ,info.reference_id
    FROM [SSISDB].[internal].[operation_messages] op LEFT JOIN [ssisdb].[internal].[event_messages] msg
           ON op.[operation_message_id] = msg.[event_message_id]
         LEFT JOIN [ssisdb].[internal].[extended_operation_info] info ON op.extended_info_id = info.info_id
    WHERE
        msg.[operation_id] = @ExecutionID
        ),
msgRef AS(
    SELECT msgEx.*, ref.[reference_type], ref.[environment_folder_name], ref.[environment_name]
    FROM msgEx LEFT JOIN [ssisdb].[internal].[environment_references] ref
    ON msgEx.[reference_id] = ref.[reference_id]
    ),
msgEnv AS(
SELECT *,
    CASE WHEN [reference_id] IS NULL THEN '-' ELSE 
    (CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name]
    END AS env
FROM
    msgRef
)
SELECT *
FROM
    msgEnv
WHERE
    (@MessageType=-1 OR [message_type] = @MessageType) AND
    (ISNULL([message_source_name],'') LIKE '%' + REPLACE(REPLACE(REPLACE(@SourceNameContains COLLATE database_default,'[', '[[]'), '_', '[_]'), '%','[%]') + '%' COLLATE database_default) AND
    (ISNULL([message],'') LIKE '%' + REPLACE(REPLACE(REPLACE(@MessageContains COLLATE database_default,'[', '[[]'), '_', '[_]'), '%','[%]') + '%' COLLATE database_default) AND
    (ISNULL([subcomponent_name],'') LIKE '%' + REPLACE(REPLACE(REPLACE(@SubComponentNameContains COLLATE database_default,'[', '[[]'), '_', '[_]'), '%','[%]') + '%' COLLATE database_default) AND
    (ISNULL([execution_path],'') LIKE '%' + REPLACE(REPLACE(REPLACE(@ExecutionPathContains COLLATE database_default,'[', '[[]'), '_', '[_]'), '%','[%]') + '%' COLLATE database_default) AND
    (ISNULL([env],'') LIKE '%' + REPLACE(REPLACE(REPLACE(@EnvironmentContains COLLATE database_default,'[', '[[]'), '_', '[_]'), '%','[%]') + '%' COLLATE database_default)
ORDER BY [message_time] DESC
END
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionErrorMsg]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionErrorMsg]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionErrorMsg]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionErrorMsg] --'63198'
	-- Add the parameters for the stored procedure here
	@ExecutionID int 

AS

WITH msgEx AS(
    SELECT
        msg.[event_message_id]
        ,msg.[operation_id]
        ,CONVERT(datetime, op.[message_time]) AS message_time
        ,op.[message_type]
        ,op.[message_source_type]
        ,CASE WHEN LEN(op.[message]) <= 4096 THEN op.[message] ELSE LEFT(op.[message], 1024) + '...' END AS [message]
        ,op.[extended_info_id]
        ,msg.[event_name]
        ,CASE WHEN LEN(msg.[message_source_name]) <= 1024 THEN msg.[message_source_name] ELSE LEFT(msg.[message_source_name], 1024) + '...' END AS [message_source_name]
        ,msg.[message_source_id]
        ,CASE WHEN LEN(msg.[subcomponent_name]) <= 1024 THEN msg.[subcomponent_name] ELSE LEFT(msg.[subcomponent_name], 1024) + '...' END AS [subcomponent_name]
        ,CASE WHEN LEN(msg.[package_path]) <= 1024 THEN msg.[package_path] ELSE LEFT(msg.[package_path], 1024) + '...' END AS [package_path]
        ,CASE WHEN LEN(msg.[execution_path]) <= 1024 THEN msg.[execution_path] ELSE LEFT(msg.[execution_path], 1024) + '...' END AS [execution_path]
        ,msg.[message_code]
        ,info.reference_id
    FROM [SSISDB].[internal].[operation_messages] op LEFT JOIN [ssisdb].[internal].[event_messages] msg
           ON op.[operation_message_id] = msg.[event_message_id]
         LEFT JOIN [ssisdb].[internal].[extended_operation_info] info ON op.extended_info_id = info.info_id
    WHERE
    msg.[operation_id] = @ExecutionID AND
    [message_type]=120
    ),
msgRef AS(
    SELECT msgEx.*, ref.[reference_type], ref.[environment_folder_name], ref.[environment_name]
    FROM msgEx LEFT JOIN [ssisdb].[internal].[environment_references] ref
    ON msgEx.[reference_id] = ref.[reference_id]
    ),
msgEnv AS(
SELECT *,
    CASE WHEN [reference_id] IS NULL THEN '-' ELSE 
    (CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name]
    END AS env
FROM
    msgRef
)
SELECT *
FROM
    msgEnv
ORDER BY [message_time] DESC
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionErrorMsgAll]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionErrorMsgAll]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionErrorMsgAll]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionErrorMsgAll] --'63198'
	-- Add the parameters for the stored procedure here
	@ExecutionID int 

AS

WITH msgEx AS(
    SELECT
        msg.[event_message_id]
        ,msg.[operation_id]
        ,CONVERT(datetime, op.[message_time]) AS message_time
        ,op.[message_type]
        ,op.[message_source_type]
        ,CASE WHEN LEN(op.[message]) <= 4096 THEN op.[message] ELSE LEFT(op.[message], 1024) + '...' END AS [message]
        ,op.[extended_info_id]
        ,msg.[event_name]
        ,CASE WHEN LEN(msg.[message_source_name]) <= 1024 THEN msg.[message_source_name] ELSE LEFT(msg.[message_source_name], 1024) + '...' END AS [message_source_name]
        ,msg.[message_source_id]
        ,CASE WHEN LEN(msg.[subcomponent_name]) <= 1024 THEN msg.[subcomponent_name] ELSE LEFT(msg.[subcomponent_name], 1024) + '...' END AS [subcomponent_name]
        ,CASE WHEN LEN(msg.[package_path]) <= 1024 THEN msg.[package_path] ELSE LEFT(msg.[package_path], 1024) + '...' END AS [package_path]
        ,CASE WHEN LEN(msg.[execution_path]) <= 1024 THEN msg.[execution_path] ELSE LEFT(msg.[execution_path], 1024) + '...' END AS [execution_path]
        ,msg.[message_code]
        ,info.reference_id
    FROM [SSISDB].[internal].[operation_messages] op LEFT JOIN [ssisdb].[internal].[event_messages] msg
           ON op.[operation_message_id] = msg.[event_message_id]
         LEFT JOIN [ssisdb].[internal].[extended_operation_info] info ON op.extended_info_id = info.info_id
    WHERE
    msg.[operation_id] = @ExecutionID
    ),
msgRef AS(
    SELECT msgEx.*, ref.[reference_type], ref.[environment_folder_name], ref.[environment_name]
    FROM msgEx LEFT JOIN [ssisdb].[internal].[environment_references] ref
    ON msgEx.[reference_id] = ref.[reference_id]
    ),
msgEnv AS(
SELECT *,
    CASE WHEN [reference_id] IS NULL THEN '-' ELSE 
    (CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name]
    END AS env
FROM
    msgRef
)
SELECT *
FROM
    msgEnv
ORDER BY [message_time] DESC
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionMsgEnvironment]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionMsgEnvironment]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionMsgEnvironment]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionMsgEnvironment] --'63198'
	-- Add the parameters for the stored procedure here
	@ExecutionID int 

AS

WITH ref AS
(
	SELECT
		[execution_id]
		,'' AS [environment_scope]
		,[reference_id]
		,[reference_type]
		,[environment_folder_name]
		,[environment_name]
	FROM
		[SSISDB].[internal].[executions]
UNION ALL
	SELECT
		v.[validation_id] AS [execution_id]
		,v.environment_scope AS [environment_scope]
		,r.[reference_id]
		,r.[reference_type]
		,r.[environment_folder_name]
		,r.[environment_name]
	FROM
		[SSISDB].[internal].[environment_references] r RIGHT JOIN [SSISDB].[internal].[validations] v ON r.[reference_id] = v.[reference_id]
)
SELECT
	CASE WHEN [environment_scope] = 'A' OR [environment_scope] = 'a' THEN 1 ELSE 0 END AS envScope,
	CASE WHEN [reference_id] IS NULL THEN '-' ELSE 
		(CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name]
	END  AS env
FROM [ref]
WHERE [execution_id] = @ExecutionID
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionMsgInfo]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionMsgInfo]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionMsgInfo]


GO
-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionMsgInfo] --'63198'
	-- Add the parameters for the stored procedure here
	@ExecutionID int 

AS

BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	WITH ref AS
	(
		SELECT
			[execution_id]
			,[reference_id]
			,[reference_type]
			,[environment_folder_name]
			,[environment_name]
		FROM
			[ssisdb].[internal].[executions]
	UNION ALL
		SELECT
			v.[validation_id] AS execution_id
			,r.[reference_id]
			,r.[reference_type]
			,r.[environment_folder_name]
			,r.[environment_name]
		FROM
			[ssisdb].[internal].[environment_references] r RIGHT JOIN [ssisdb].[internal].[validations] v ON r.[reference_id] = v.[reference_id]
	),

	ve AS
	(SELECT [operation_type],
			[validation_id] AS execution_id, 
			[use32bitruntime], 
			[start_time], 
			[end_time], 
			[caller_name], 
			[folder_name], 
			[project_name], 
			[object_name] as package_name, 
			ENUM.[Description] as [status], 
			'v' AS op_type 
	FROM [ssisdb].[internal].[validations] v 
		INNER JOIN [ssisdb].[internal].[operations] o ON v.[validation_id] = o.[operation_id]
		INNER JOIN [RTLS_NDRDW].[ssisOperationStatus] ENUM on o.[status] = ENUM.[Value]
	WHERE [object_type] = 30 OR [object_type] = 20
	UNION ALL
	SELECT	[operation_type],
			[execution_id], 
			[use32bitruntime], 
			[start_time], 
			[end_time], 
			[caller_name], 
			[folder_name], 
			[project_name], 
			[package_name], 
			ENUM.[Description] as [status], 
			'e' AS op_type 
	FROM [ssisdb].[internal].[executions] e 
	INNER JOIN [ssisdb].[internal].[operations] o ON e.[execution_id]= o.[operation_id]
	INNER JOIN  [RTLS_NDRDW].[ssisOperationStatus] ENUM on o.[status] = ENUM.[Value]
	)

	SELECT	[operation_type]
			,[execution_id]
			,[folder_name]
			,[project_name]
			,[package_name]
			,[use32bitruntime]
			,[status]
			,CONVERT (datetime, [start_time]) AS start_time
			,CONVERT (datetime, [end_time]) AS end_time
			,CONVERT(FLOAT, DATEDIFF(millisecond, [start_time], ISNULL([end_time], SYSDATETIMEOFFSET())))/1000 AS duration
			,[caller_name]
			,[op_type]
			, (SELECT CASE WHEN [reference_id] IS NULL 
				THEN '-' ELSE (CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' 
				THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name] 
				END  AS env FROM [ref] WHERE [execution_id] = @ExecutionID) as env
	FROM [ve]
	WHERE
	[execution_id] = @ExecutionID

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionStatistics]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionStatistics]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionStatistics]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04/15/2015
-- Description:	Gets ETL Job execution stats 
--				from ETL framework data
--				for ETL Dashboard Report
-- Parameters:	@StartDate: start date of data pull
--				@EndDate: end date of data pull
-- =============================================

CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionStatistics] --'04/01/2015', '04/16/2016'
@StartDate as DateTime,
@EndDate as DateTime

AS
BEGIN

WITH Stat as 
(
SELECT DISTINCT
	  [a].[ExecutionJobLogID]
	, DATEDIFF( SECOND, [c].[ExecutionJobLogStart], [c].[ExecutionJobLogStop]) AS [ExecutionJobDurationSeconds]
	, SUM([a].[ExecutionLogRowsProcessed]) as RowsProcessed
	, SUM([a].[ExecutionLogBytesTransferred]) as BytesTransferred
	, COUNT( [a].[ExecutionLogID] ) AS [TotalCount]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 0 THEN 1 ELSE 0 END ) AS [Pending]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 2 THEN 1 ELSE 0 END ) AS [TotalSuccess]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 3 THEN 1 ELSE 0 END ) AS [TotalError]
	, SUM( CASE WHEN CAST([c].[ExecutionJobLogStop] AS TIME) > CAST([s].[ScheduleStopTime] AS TIME) THEN 1 ELSE 0 END) AS [JobsOutsideWindow]
FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
	JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
	JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
	JOIN [RTLS_NDRDW].[ScheduleMaster] [s] on [b].[ScheduleMasterID] = [s].[ScheduleMasterID]
WHERE [c].[ExecutionJobLogStart] BETWEEN @StartDate AND @EndDate
group by [a].[ExecutionJobLogID], [c].[ExecutionJobLogStart], [c].[ExecutionJobLogStop]
)

SELECT
	  AVG([ExecutionJobDurationSeconds]) as AvgTime
	, MIN([ExecutionJobDurationSeconds]) as MinTime
	, MAX([ExecutionJobDurationSeconds]) as MaxTime
	, SUM(RowsProcessed) as RowsProcessed
	, SUM(BytesTransferred) as BytesTransferred
	, SUM([TotalCount]) as TotalCount
	, SUM([JobsOutsideWindow]) as JobsOutsideWindow
	, IIF(SUM(BytesTransferred) = 0, 0, (SUM(BytesTransferred) / CAST(AVG([ExecutionJobDurationSeconds]) AS FLOAT))) as AvgBytesPerSec 
	, SUM([Pending]) as JobsPending
	, (SUM(CAST([TotalSuccess] AS FLOAT)) / SUM(CAST([TotalError] AS FLOAT))) *.1 as SuccessRatio
FROM Stat


END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptExecutionStatusReport]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptExecutionStatusReport]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptExecutionStatusReport]
GO


--
--
--
CREATE PROCEDURE [RTLS_NDRDW].[rptExecutionStatusReport] --'04/01/2015', '04/16/2015'
@StartDate as DateTime,
@EndDate as DateTime

AS
BEGIN
SELECT 
	[a].[ExecutionJobLogID]
	, [c].[ExecutionJobLogStart]
	, [b].[JobMasterID]
	, [b].[JobMasterName]

	, CASE WHEN ([c].[ExecutionJobLogStop] IS NULL) THEN DATEDIFF( SECOND, [c].[ExecutionJobLogStart], GETDATE() )
		ELSE DATEDIFF( SECOND, [c].[ExecutionJobLogStart], [c].[ExecutionJobLogStop] )
	END AS [ExecutionJobDurationSeconds]

	, [c].[ExecutionJobLogStatusID]


	, COUNT( [a].[ExecutionLogID] ) AS [TotalCount]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 0 THEN 1 ELSE 0 END ) AS [TotalNotRun]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 1 THEN 1 ELSE 0 END ) AS [TotalRunning]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 2 THEN 1 ELSE 0 END ) AS [TotalSuccess]
	, SUM( CASE [a].[ExecutionLogStatusID] WHEN 3 THEN 1 ELSE 0 END ) AS [TotalError]
FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
	JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
	JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
WHERE [c].[ExecutionJobLogStart] BETWEEN @StartDate AND @EndDate

GROUP BY
	[a].[ExecutionJobLogID]
	, [c].[ExecutionJobLogStart]
	, [b].[JobMasterID]
	, [b].[JobMasterName]
	, [c].[ExecutionJobLogStop]
	, [c].[ExecutionJobLogStatusID]

ORDER BY
	[a].[ExecutionJobLogID]
	, [c].[ExecutionJobLogStart]
	, [b].[JobMasterID]
	, [b].[JobMasterName]
	, [c].[ExecutionJobLogStatusID]

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptGetCPUUtilization]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptGetCPUUtilization]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptGetCPUUtilization]
GO


CREATE Procedure [RTLS_NDRDW].[rptGetCPUUtilization]
AS
BEGIN

SELECT	EventTime
		,SystemCPUUtilization
		,SQLCPUUtilization
		,SystemIdleProcess
		,TotalCPUUtilization
FROM            [RTLS_NDRDW].[rptCPUUtilizationHistory]
WHERE        (EventTime BETWEEN DATEADD(HOUR, -2, GETDATE()) AND GETDATE())

END
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptGetMemoryUtilization]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptGetMemoryUtilization]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptGetMemoryUtilization]
GO


CREATE Procedure [RTLS_NDRDW].[rptGetMemoryUtilization]
AS
BEGIN

SELECT  EventTime
		, TotalPhysMemMB
		, AvailPhysMemMB
		, TotalPageFileMB
		, AvailPageFileMB
		, system_memory_state_desc
FROM [RTLS_NDRDW].[rptMemoryUtilizationHistory]
WHERE (EventTime BETWEEN DATEADD(HOUR, -2, GETDATE()) AND GETDATE())

END
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptJobCount]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptJobCount]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptJobCount]
GO


-- =============================================
-- Author:		Michael D. Hanson
--				HP Enterprise Services
-- Create date: 04-15-2015
-- Description:	This procedure calculates 30 day 
--				job counts for the ETL Dashboard
-- Parameters:	none
-- =============================================

CREATE Procedure [RTLS_NDRDW].[rptJobCount] 
AS
BEGIN


WITH RunCount AS
(
SELECT	DISTINCT [a].[ExecutionJobLogID] as [RunExecutionID]
		, CASE [a].[ExecutionLogStatusID]
			WHEN 0 THEN 'Pending'
			WHEN 1 THEN 'Running'
			WHEN 2 THEN 'Success'
			WHEN 3 THEN 'Error'
		END as [RunStatusName]
		,COUNT([a].[ExecutionJobLogID]) as [RunCount]
		,Convert(Date, [c].[ExecutionJobLogStart]) as RunStartDateTime
FROM [RTLS_NDRDW].[ExecutionLog] [a] WITH (NOLOCK)
		JOIN [RTLS_NDRDW].[JobMaster] [b] ON [a].[JobMasterID] = [b].[JobMasterID]
		JOIN [RTLS_NDRDW].[ExecutionJobLog] [C] ON [a].[ExecutionJobLogID] = [C].[ExecutionJobLogID]
WHERE (Convert(DateTime, [c].[ExecutionJobLogStart]) BETWEEN Getdate()-30 AND GetDate()) 
GROUP BY [a].[ExecutionJobLogID], [a].[ExecutionLogStatusID], Convert(Date, [c].[ExecutionJobLogStart])
),  DateRanges AS
(
SELECT Convert(Date, Getdate()-30) AS DateValue
UNION ALL
SELECT DATEADD(DAY, 1, DateValue)
FROM DateRanges
WHERE DateValue < Convert(Date, GetDate())
)
SELECT	d.DateValue
		,r.[RunExecutionID]
		,r.[RunStatusName]	
		,ISNULL(SUM(r.[RunCount]), 0) as runs
from [RunCount] r right outer join [DateRanges] d on r.RunStartDateTime = d.DateValue
group by d.DateValue, r.[RunExecutionID], r.[RunStatusName]
order by d.[DateValue], r.[RunExecutionID]

END
GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptOverviewExecutionInfo]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptOverviewExecutionInfo]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptOverviewExecutionInfo]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:  @ovrExecutionID: execution ID
--				for SSISDB catalog information
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptOverviewExecutionInfo] --'62502'
	-- Add the parameters for the stored procedure here
	@ovrExecutionID int 

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	WITH ref AS
	(
		SELECT
			[execution_id]
			,[reference_id]
			,[reference_type]
			,[environment_folder_name]
			,[environment_name]
		FROM
			[ssisdb].[internal].[executions]
	UNION ALL
		SELECT
			v.[validation_id] AS execution_id
			,r.[reference_id]
			,r.[reference_type]
			,r.[environment_folder_name]
			,r.[environment_name]
		FROM
			[ssisdb].[internal].[environment_references] r RIGHT JOIN [ssisdb].[internal].[validations] v ON r.[reference_id] = v.[reference_id]
	),

	ve AS
	(SELECT [validation_id] AS execution_id, 
			[use32bitruntime], 
			[start_time], 
			[end_time], 
			[caller_name], 
			[folder_name], 
			[project_name], 
			[object_name] as package_name, 
			ENUM.[Description] as [status], 
			'v' AS op_type 
	FROM [ssisdb].[internal].[validations] v 
		INNER JOIN [ssisdb].[internal].[operations] o ON v.[validation_id] = o.[operation_id]
		INNER JOIN [RTLS_NDRDW].[ssisOperationStatus] ENUM on o.[status] = ENUM.[Value]
	WHERE [object_type] = 30
	UNION ALL
	SELECT [execution_id], 
		[use32bitruntime], 
		[start_time], 
		[end_time], 
		[caller_name], 
		[folder_name], 
		[project_name], 
		[package_name], 
		ENUM.[Description] as [status], 
		'e' AS op_type 
	FROM [ssisdb].[internal].[executions] e 
	INNER JOIN [ssisdb].[internal].[operations] o ON e.[execution_id]= o.[operation_id]
	INNER JOIN  [RTLS_NDRDW].[ssisOperationStatus] ENUM on o.[status] = ENUM.[Value]
	)

	SELECT	[execution_id]
			,[folder_name]
			,[project_name]
			,[package_name]
			,[use32bitruntime]
			,[status]
			,CONVERT (datetime, [start_time]) AS start_time
			,CONVERT (datetime, [end_time]) AS end_time
			,CONVERT(FLOAT, DATEDIFF(millisecond, [start_time], ISNULL([end_time], SYSDATETIMEOFFSET())))/1000 AS duration
			,[caller_name]
			,[op_type]
			, (SELECT CASE WHEN [reference_id] IS NULL 
				THEN '-' ELSE (CASE WHEN [reference_type] = 'R' OR [reference_type] = 'r' 
				THEN '.' ELSE [environment_folder_name] END) + '\' + [environment_name] 
				END  AS env FROM [ref] WHERE [execution_id] = @ovrExecutionID) as env
	FROM [ve]
	WHERE
	[execution_id] = @ovrExecutionID

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptOverviewExecutionOverview]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptOverviewExecutionOverview]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptOverviewExecutionOverview]


GO
-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04-15-2015
-- Description:	Gets SSISDB Parameter Info 
--				for ETL Dashboard Report
-- Parameters:	@ovrExecutionID: Execution ID
--				used to pull information from
--				SSISDB Catalog
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptOverviewExecutionOverview] --'62520'
	-- Add the parameters for the stored procedure here
	@ovrExecutionID int 

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	SELECT 	executable_statistics.execution_id,
			executables.executable_id, 
		CASE 
			WHEN LEN(executables.[executable_name]) <= 1024 
			THEN executables.[executable_name] 
			ELSE LEFT(executables.[executable_name], 1024) + '...' 
		END AS executable_name, 
		executables.executable_guid, 
                         
		CASE 
			WHEN LEN(executable_statistics.[execution_path]) <= 1024 
			THEN executable_statistics.[execution_path] 
			ELSE LEFT(executable_statistics.[execution_path], 1024) + '...' 
		END AS execution_path, 
		executables.package_name, 
		executable_statistics.statistics_id, 
		CONVERT(FLOAT, executable_statistics.execution_duration) / 1000 AS execution_duration, 
		executable_statistics.execution_result, 
		executable_statistics.execution_value
	FROM	[SSISDB].[internal].[executable_statistics] AS executable_statistics 
		INNER JOIN [SSISDB].[internal].[executables] AS executables 
		ON --executable_statistics.execution_id = executables.execution_id 
		--AND 
		executable_statistics.executable_id = executables.executable_id
	WHERE	(executable_statistics.execution_id = @ovrExecutionID) 
		/*AND (ISNULL(executables.package_name, '') 
			LIKE '%' + REPLACE(REPLACE(REPLACE(@ovrPackageNameContains COLLATE database_default, '[', '[[]'), '_', '[_]'), '%', '[%]') + '%' COLLATE database_default) 
		AND (ISNULL(executable_statistics.execution_path, N'') 
			LIKE '%' + REPLACE(REPLACE(REPLACE(@ovrExecutionPathContains COLLATE database_default, '[', '[[]'), '_', '[_]'), '%', '[%]') + '%' COLLATE database_default) 
		AND (ISNULL(executables.executable_name, N'') 
					LIKE '%' + REPLACE(REPLACE(REPLACE(@ovrExecutableNameContains COLLATE database_default, '[', '[[]'), '_', '[_]'), '%', '[%]') + '%' COLLATE database_default) 
		AND (executable_statistics.execution_result = (CASE WHEN (@ovrResult > - 1) THEN @ovrResult ELSE executable_statistics.[execution_result] END))*/
	ORDER BY executables.executable_id

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptOverviewParameter]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptOverviewParameter]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptOverviewParameter]


GO
-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04/15/2015
-- Description:	Gets SSISDB Parameter Info 
--				from SSISDB Catalog for use in
--				ETL Dashboard Report
-- Parameters:	@ovrExecutionID: Execution ID
--				assosciated with SSISDB Catalog
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptOverviewParameter] --'62520'
	-- Add the parameters for the stored procedure here
	@ovrExecutionID int 

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	SELECT
	   [execution_parameter_id]
	  ,[execution_id]
	  ,[object_type]
	  ,[parameter_data_type]
	  ,[parameter_name] = 
	   CASE
		  WHEN [parameter_name] LIKE 'CM.%' THEN SUBSTRING([parameter_name], 4, 4000)
		  ELSE [parameter_name]
	   END
	  ,[parameter_value] =
	   CASE sensitive
		 WHEN 1 THEN '********'
		 ELSE [parameter_value]
	   END
	  ,[sensitive]
	  ,[required]
	  ,[value_set]
	  ,[runtime_override]
	FROM
	  [SSISDB].[internal].[execution_parameter_values]
	WHERE
	  [execution_id] = @ovrExecutionID AND
	  ([value_set] = 1 OR [parameter_name] NOT LIKE 'CM.%')
	ORDER BY [parameter_name]

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptOverviewPropertyOverride]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptOverviewPropertyOverride]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptOverviewPropertyOverride]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 04/15/2015
-- Description:	Gets SSISDB Properties Info 
--				from SSISDB Catalog
--				for ETL Dashboard Report
-- Parameters:	@ovrExecutionID: Execution ID
--				associated with SSISDB Catalog
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptOverviewPropertyOverride] --'62520'

	@ovrExecutionID int 

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	SELECT
	   [property_path]
	  ,[property_value] =
	   CASE sensitive
		 WHEN 1 THEN '********'
		 ELSE [property_value]
	   END
	FROM 
	  [SSISDB].[internal].[execution_property_override_values]
	WHERE 
	  [execution_id] = @ovrExecutionID
	ORDER BY 
	  [property_path]

END

GO
/****** Object:  StoredProcedure [RTLS_NDRDW].[rptParamSSISDBFolders]    Script Date: 5/8/2015 2:00:21 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('[RTLS_NDRDW].[rptParamSSISDBFolders]', 'P') IS NOT NULL
DROP PROC [RTLS_NDRDW].[rptParamSSISDBFolders]
GO


-- =============================================
-- Author:		Michael D. Hanson
-- Create date: 01/08/2015
-- Description:	Get folder details from SSISDB Catalog
-- =============================================
CREATE PROCEDURE [RTLS_NDRDW].[rptParamSSISDBFolders] 
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	SELECT  name
	FROM    [SSISDB].[internal].[folders]
	where name like 'RTLS%'
	order by name
END

GO
